#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2026 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')

# Reuse the same design; only the optimization switch differs.
test.top_filename = "t/t_bit_scan_loops.v"

test.compile(verilator_flags2=['--stats', '--unroll-count', '0', '-fno-bit-scan-loops'])

# With the optimization disabled, nothing lowers.
test.file_grep(test.stats, r'Lowered priority-encoder to mostsetbitp1\s+([0-9])', 0)

test.execute()

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')

test.run(cmd=[os.environ["VERILATOR_ROOT"] + "/bin/verilator_coverage", '--invalid-dash'],
         logfile=test.run_log_filename,
         fails=True,
         expect_filename=test.golden_filename,
         verilator_run=True)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('simulator')
test.top_filename = "t/t_benchmark_mux4k.v"

test.compile(v_flags2=["--stats", test.wno_unopthreads_for_few_cores])

# WSL2 gives a warning and we must skip the test:
#  "physcpubind: 0 1 2 3 ...\n No NUMA support available on this system."
nout = test.run_capture("numactl --show", check=False)

if not nout or not re.search(r'cpu', nout) or re.search(r'No NUMA support available', nout,
                                                        re.IGNORECASE):
    test.skip("No numactl available")

test.execute(run_env='numactl -m 0 -C 0,0,0,0,0,0,0,0')

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios("vlt_all")

test.compile(v_flags2=[
    "--trace-vcd --trace-max-width 0 --trace-max-array 0 --output-split-ctrace 10 --trace-structs"
])
trace_files = glob.glob(test.obj_dir + "/*Trace*.cpp")
if len(trace_files) < 10:
    test.error("Too few trace files")

test.execute()

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap
import collections

test.scenarios('vlt')


def preproc_check(filename1, filename2):
    # Read line comments.
    line_checks = collections.deque()
    with open(filename1, 'r', encoding="latin-1", newline='\n') as fh:
        lineno = 0
        for line in fh:
            lineno += 1
            if re.match(r'^Line_Preproc_Check', line):
                line_checks.append(lineno)

    # See if output file agrees.
    with open(filename2, 'r', encoding="latin-1", newline='\n') as fh:
        lineno = 0
        for line in fh:
            lineno += 1
            m = re.match(r'^\`line\s+(\d+)', line)
            if m:
                lineno = int(m.group(1)) - 1
            m = re.match(r'^Line_Preproc_Check\s+(\d+)', line)
            if m:
                linecmt = m.group(1)
                check = line_checks.popleft()
                file2ln = filename2 + ":" + str(lineno)
                if not check:
                    test.error(file2ln + ": Extra Line_Preproc_Check")
                if str(linecmt) != str(check):
                    test.error(file2ln + ": __LINE__ inserted " + str(linecmt) + ", exp=" +
                               str(check))
                if str(lineno) != str(check):
                    test.error(file2ln + ": __LINE__ on `line " + str(lineno) + ", exp=" +
                               str(check))

    if len(line_checks):
        test.error(filename2 + ": Missing a Line_Preproc_Check")


stdout_filename = test.obj_dir + "/" + test.name + "__test.vpp"

test.compile(verilator_flags2=['-DDEF_A0 -DPREDEF_COMMAND_LINE -E'],
             verilator_make_gmake=False,
             make_top_shell=False,
             make_main=False,
             stdout_filename=stdout_filename)

preproc_check(test.top_filename, stdout_filename)
test.files_identical(stdout_filename, test.golden_filename)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')

test.setenv("ASAN_OPTIONS", "handle_segv=0")
test.leak_check_disable()

test.lint(v_flags=["--debug-sigsegv"], fails='any')

test.file_grep(test.compile_log_filename,
               r'%Error: Verilator internal fault, sorry. Suggest trying --debug --gdbbt')
test.file_grep(test.compile_log_filename, r'Command Failed')

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('dist')

if not os.path.exists(test.root + "/.git"):
    test.skip("Not in a git repository")

test.run(cmd=[
    "cd " + test.root + "/src/obj_dbg && " + os.environ['MAKE'] +
    " -j 4 -k -f ../Makefile_obj VL_NOOPT=1 header_cc"
],
         check_finished=False)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2026 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import os
import sys
import vltest_bootstrap

test.scenarios('vltmt')

test.top_filename = "t/t_gantt.v"
test.pli_filename = "t/t_gantt_numa_default_threads.cpp"

# Require enough cores so default thread count stays >= model threads
# (we don't call contextp->threads in this test)
test.skip_if_too_few_cores()

test.compile(
    make_main=False,
    verilator_flags2=[
        "--prof-exec",
        "--exe",
        test.pli_filename,
        test.t_dir + "/t_gantt_c.cpp",
    ],
    threads=test.get_default_vltmt_threads,
)

test.execute(all_run_flags=[
    "+verilator+prof+exec+start+2",
    " +verilator+prof+exec+window+2",
    " +verilator+prof+exec+file+" + test.obj_dir + "/profile_exec.dat",
])

gantt_log = test.obj_dir + "/gantt_default_threads.log"
test.run(cmd=[
    os.environ["VERILATOR_ROOT"] + "/bin/verilator_gantt",
    "--no-vcd",
    test.obj_dir + "/profile_exec.dat",
    "| tee " + gantt_log,
])

if sys.platform != "darwin":
    test.file_grep(gantt_log, r"NUMA status += NUMA assignment not requested")

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2025 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios("simulator_st")

test.compile(timing_loop=True, verilator_flags2=["--timing"])

filename = test.obj_dir + "/bits.log"

test.unlink_ok(filename)

for seed in range(1, 6):
    test.execute(all_run_flags=["+verilator+rand+reset+2", f"+verilator+seed+{seed}"])

unique_strings = set()

with open(filename, "r", encoding="utf-8") as file:
    for line in file:
        unique_strings.update(line.split())

count_unique_strings = len(unique_strings)

# A ideal random generator has a fail chance of 1 in 66 million
expected_unique_strings = 4
if count_unique_strings < expected_unique_strings:
    test.error(
        f"Expected at least {expected_unique_strings} unique strings, got {count_unique_strings}")

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')
test.top_filename = "t/t_unroll_pragma.v"

test.compile(verilator_flags2=['--unroll-count 4 --unroll-stmts 9999 --stats -DTEST_NONE'],
             verilator_make_gmake=False,
             make_top_shell=False,
             make_main=False)

test.file_grep(test.stats, r'Optimizations, Loop unrolling, Unrolled loops\s+(\d+)', 5)
test.file_grep(test.stats, r'Optimizations, Loop unrolling, Unrolled iterations\s+(\d+)', 25)
test.file_grep(test.stats,
               r'Optimizations, Loop unrolling, Failed - reached --unroll-count\s+(\d+)', 7)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')
test.top_filename = "t/t_hier_block.v"

test.lint(
    fails=True,
    verilator_flags2=[
        '--hierarchical-block',
        'modName,mangledName,param0,"paramValue0",param0,"paramValue1",param1,2,param3',
        '--hierarchical-block', 'modName',
        '--hierarchical-block', 'mod0,mod1,\'"str\\\'',  # end with backslash
        '--hierarchical-block', 'mod2,mod3,\'"str\\a\'',  # unexpected 'a' after backslash
        '--hierarchical-block', 'mod4,mod5,\'"str"abc\',',  # not end with "
        '--hierarchical-block', 'mod6,mod7,\'"str"\',',  # end with ,
        '--hierarchical-block', 'mod8,mod9,\'s"tr"\',',  # unexpected "
        '--hierarchical-block', 'modA,modB,param,',  # end with ,
    ],
    expect_filename=test.golden_filename)  # yapf:disable

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')
test.top_filename = os.path.join(test.obj_dir, "t_inst_long.v")

length = 200
longname = "long_" * int((length + 4) / 5)


def gen_top(filename):
    with open(filename, 'w', encoding="utf8") as fh:
        fh.write("// Generated by t_inst_long.py\n")
        fh.write("module t;\n")
        fh.write("\n")
        fh.write("  " + longname + " inst ();\n")
        fh.write("\n")
        fh.write("endmodule\n")


def gen_sub(filename):
    with open(filename, 'w', encoding="utf8") as fh:
        fh.write("// Generated by t_inst_long.py\n")
        fh.write("module " + longname + ";\n")
        fh.write("\n")
        fh.write("  initial begin\n")
        fh.write("     $write(\"*-* All Finished *-*\\n\");\n")
        fh.write("     $finish;\n")
        fh.write("  end\n")
        fh.write("endmodule\n")


gen_top(test.top_filename)
gen_sub(os.path.join(test.obj_dir, longname + ".v"))

test.compile()

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2026 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')

test.compile(verilator_flags2=["--stats"])

memUsageMB = int(test.file_grep(test.stats, r'Peak Memory Usage \(MB\) +(\d+)')[0])

if memUsageMB > 128 and not test.have_dev_asan:
    test.error("Consumed over 128MB memory")

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt_all')
test.top_filename = "t/t_timing_class.v"

# Disable CFunc inlining so debug traces show all function entries
test.compile(verilator_flags2=["--exe --main --timing -fno-inline-cfuncs"])

test.execute(all_run_flags=["+verilator+debug"])

if not test.vltmt:  # vltmt output may vary between thread exec order
    test.files_identical(test.obj_dir + "/vlt_sim.log", test.golden_filename, "logfile")

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('simulator')
test.top_filename = "t/t_func_dotted.v"
out_filename = test.obj_dir + "/V" + test.name + ".tree.json"

test.compile(v_flags2=["--no-json-edit-nums", "t/t_func_dotted_inl2.vlt"])

if test.vlt_all:
    modps = test.file_grep(
        out_filename,
        r'{"type":"MODULE","name":"mb","addr":"([^"]*)","loc":"\w,99:[^"]*",.*"origName":"mb"')
    modp = modps[0]
    test.file_grep(
        out_filename,
        r'{"type":"CELL","name":"t.ma0.mb0","addr":"[^"]*","loc":"\w,87:[^"]*",.*"origName":"mb0",.*"modp":"([^"]*)"',
        modp)

test.execute()

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.priority(30)
test.scenarios('vlt_all', 'xsim')
test.top_filename = "t/t_hier_block.v"

secret_prefix = "secret"
secret_dir = test.obj_dir + "/" + secret_prefix
test.mkdir_ok(secret_dir)
abs_secret_dir = os.path.abspath(secret_dir)

# Always compile the secret file with Verilator no matter what simulator
#   we are testing with
test.run(logfile=secret_dir + "/vlt_compile.log",
         cmd=[
             "perl", os.environ["VERILATOR_ROOT"] + "/bin/verilator", "-cc", "--hierarchical",
             "-Mdir", secret_dir, "--protect-lib", secret_prefix, "--protect-key", "PROTECT_KEY",
             "t/t_hier_block.v", "-DAS_PROT_LIB", '--CFLAGS', '"-pipe -DCPP_MACRO=cplusplus"',
             (' --threads 1' if test.vltmt else ''), "--build"
         ],
         verilator_run=True)

test.compile(
    v_flags2=['t/t_hier_block.cpp'],
    verilator_flags2=[
        secret_dir + "/secret.sv", "-DPROTLIB_TOP", "--top-module t", "-LDFLAGS",
        "'-Wl,-rpath," + abs_secret_dir + " -L" + abs_secret_dir + " -l" + secret_prefix + "'"
    ])

test.execute(run_env="DYLD_FALLBACK_LIBRARY_PATH=" + abs_secret_dir)

test.file_grep(secret_dir + "/Vsub0/sub0.sv", r'^module\s+(\S+)\s+', "sub0")
test.file_grep(secret_dir + "/Vsub1/sub1.sv", r'^module\s+(\S+)\s+', "sub1")
test.file_grep(secret_dir + "/Vsub2/sub2.sv", r'^module\s+(\S+)\s+', "sub2")
test.file_grep(test.run_log_filename, r'MACRO:(\S+) is defined', "cplusplus")

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('simulator')
test.pli_filename = "t/t_vpi_var.cpp"

test.compile(make_top_shell=False,
             make_main=False,
             make_pli=True,
             sim_time=2100,
             iv_flags2=["-g2005-sv -D USE_VPI_NOT_DPI -DWAVES -DT_VPI_VAR2"],
             v_flags2=["+define+USE_VPI_NOT_DPI"],
             verilator_flags2=["-Wno-SYMRSVDWORD --exe --vpi --no-l2name", test.pli_filename])

test.execute(use_libvpi=True, all_run_flags=['+PLUS +INT=1234 +STRSTR'])

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.priority(30)
test.scenarios('vlt_all')

# stats will be deleted but generation will be skipped if libs of hierarchical blocks exist.
test.clean_objs()

test.setenv('TEST_ROOT', test.t_dir + "/t_hier_block_import")

# CI environment offers 2 VCPUs, 2 thread setting causes the following warning.
# %Warning-UNOPTTHREADS: Thread scheduler is unable to provide requested parallelism; consider asking for fewer threads.
# So use 6 threads here though it's not optimal in performance, but ok.

test.compile(verilator_flags2=[
    '$TEST_ROOT/t_hier_block_import_def.vh', '-f $TEST_ROOT/t_hier_block_import_args.vc',
    '-I$TEST_ROOT'
],
             threads=(6 if test.vltmt else 1))

test.execute()

test.file_grep(test.obj_dir + "/VsubA/subA.sv", r'^module\s+(\S+)\s+', "subA")
test.file_grep(test.stats, r'HierBlock,\s+Hierarchical blocks\s+(\d+)', 2)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2025 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')

# +verilator+vpi+ pointing at a non-existent library must fail with a clear
# error (the dlopen-failure branch of the runtime loader).
test.top_filename = 't/t_flag_main_vpi.v'

test.compile(verilator_flags2=["--binary --vpi --public-flat-rw"])

# The fatal names the (stable, relative) library path; the platform-specific dlerror()
# detail is emitted on a "- " line, which golden comparison strips, so the .out is portable.
test.execute(fails=True,
             all_run_flags=["+verilator+vpi+" + test.obj_dir + "/nonexistent.so"],
             expect_filename=test.golden_filename)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2026 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap
import coverage_covergroup_common

test.scenarios('vlt')

coverage_covergroup_common.run(test)

# Verify coverage.dat format contains covergroup entries (replaces t_covergroup_database)
test.file_grep(test.coverage_filename, r'covergroup')
test.file_grep(test.coverage_filename, r'bin.{0,2}low')
test.file_grep(test.coverage_filename, r'bin.{0,2}high')
test.file_grep(test.coverage_filename, r'cg_db\.cp\.low')
test.file_grep(test.coverage_filename, r'cg_db\.cp\.high')
test.file_grep(test.coverage_filename, r'.*bin.{0,2}low.*\' [1-9]')
test.file_grep(test.coverage_filename, r'.*bin.{0,2}high.*\' [1-9]')



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('dist')

if not os.path.exists(test.root + "/.git"):
    test.skip("Not in a git repository")

### Must trim output before and after our file list
warns = {}
summary = None

status = test.run_capture("cd " + test.root + " && git ls-files -o --exclude-standard")

if test.verbose:
    print("-ST " + status)
for filename in sorted(status.split()):
    if re.search('nodist', filename):
        continue
    warns[filename] = "File not in git or .gitignore: " + filename
    summary = "Files untracked in git or .gitignore:"

if summary:
    # First warning lists everything as that's shown in the driver summary
    test.error(summary + " " + ' '.join(sorted(warns.keys())))
    for filename in sorted(warns.keys()):
        test.error(warns[filename])

test.passes()



# pylint: disable=R0914
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2026 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import re


def run(test, *, verilator_flags2=()):
    variant, mode, fmt = test.parse_name(
        r"t_trace_hier_block_(default|notop|statefulpkg)_(cc|sc)_([a-z]+)")

    # All test use the same SV file
    test.top_filename = "t/t_hier_block.v"
    # Any variations after the format name must yield the exact same trace
    test.golden_filename = test.py_filename.rpartition(fmt)[0] + fmt + ".out"

    verilator_common_flags = []
    match mode:
        case "cc":
            verilator_common_flags.extend([  #
                "--cc",
            ])
        case "sc":
            verilator_common_flags.extend([  #
                "--sc",
                "--CFLAGS",
                '"-pipe -DCPP_MACRO=cplusplus"',
            ])
        case _:
            test.error(f"Unknown mode '{mode}'")

    verilator_common_flags.extend([
        f"--trace-{fmt}",
        "--trace-underscore",  # Should not trace __Vhandle
        "--trace-max-width",
        "0",
        "--trace-max-array",
        "0",
        "--trace-structs",
        "t/t_hier_block.cpp"
    ])

    main_top_name = "top"
    match variant:
        case "default":
            pass
        case "notop":
            main_top_name = ""
        case "statefulpkg":
            verilator_common_flags.append("+define+STATEFUL_PKG")
        case _:
            test.error(f"Unhandled test variant '{variant}'")

    verilator_common_flags.extend(verilator_flags2)
    verilator_hier_flags = verilator_common_flags + ['--hierarchical']

    # Compile hierarchically
    test.vm_prefix = "Vhier"
    test.main_filename = test.obj_dir + "/Vhier__main.cpp"
    test.compile(verilator_flags2=verilator_hier_flags, main_top_name=main_top_name)

    # Compile non-hierarchically
    test.vm_prefix = "Vnonh"
    test.main_filename = test.obj_dir + "/Vnonh__main.cpp"
    test.compile(verilator_flags2=verilator_common_flags, main_top_name=main_top_name)

    trace_hier = test.trace_filename.replace("simx", "hier")
    trace_nonh = test.trace_filename.replace("simx", "nonh")

    # Run the hierarchical model
    test.execute(executable=test.obj_dir + "/Vhier")
    test.run(cmd=["mv", test.trace_filename, trace_hier])
    # Run the non hierarchical model
    test.execute(executable=test.obj_dir + "/Vnonh")
    test.run(cmd=["mv", test.trace_filename, trace_nonh])

    if variant != "statefulpkg":
        # Scope structure must match exactly, check only in vcd
        if fmt == "vcd":
            with open(trace_nonh, 'r', encoding='utf8') as fnonh, \
                 open(trace_hier, 'r', encoding='utf8') as fhier:
                for la, lb in zip(fnonh, fhier):
                    la = re.sub(r'^(\s*\$var\s+\S+\s+\S+\s+)\S+(.*)$', r'\1CODE\2', la)
                    lb = re.sub(r'^(\s*\$var\s+\S+\s+\S+\s+)\S+(.*)$', r'\1CODE\2', lb)
                    if la != lb:
                        test.error_keep_going("VCD header mismatch: '{}' !~ '{}'".format(
                            la.strip(), lb.strip()))
                    if "enddefinitions" in la:
                        break

        test.trace_identical(trace_hier, trace_nonh)
    # The hierarchical must match the reference
    test.trace_identical(trace_hier, test.golden_filename)

    test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('simulator')
test.top_filename = "t/t_param_in_func.v"

test.compile(verilator_flags2=["--stats", "+define+NO_INLINE=1"])

test.execute()

# The parameter array should have been put in the constant pool
if test.vlt_all:
    test.file_grep(test.stats, r'ConstPool, Tables emitted\s+(\d+)', 3)

# Shouldn't have any references to the parameter array
for filename in (test.glob_some(test.obj_dir + "/" + test.vm_prefix + "*.h") +
                 test.glob_some(test.obj_dir + "/" + test.vm_prefix + "*.cpp")):
    test.file_grep_not(filename, r'digits')

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vltmt')

test.compile(verilator_flags2=['--cc'], threads=4, context_threads=2)

test.execute(fails=True)

test.file_grep(
    test.run_log_filename,
    r"%Error: .*\/verilated\.cpp:\d+: VerilatedContext has 2 threads but model 'Vt_threads_crazy' \(instantiated as 'top'\) was Verilated with --threads 4\."
)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.priority(30)
test.scenarios('dist')
test.rerunnable = False


def have_clang_check():
    cmd = 'python3 -c "from clang.cindex import Index; index = Index.create(); print(\\"Clang imported\\")";'
    if test.verbose:
        print("\t" + cmd)
    nout = test.run_capture(cmd, check=False)
    if not nout or not re.search(r'Clang imported', nout):
        return False
    return True


if 'VERILATOR_TEST_NO_ATTRIBUTES' in os.environ:
    test.skip("Skipping due to VERILATOR_TEST_NO_ATTRIBUTES")
if not os.path.exists(test.root + "/src/obj_dbg/compile_commands.json"):
    test.skip("compile_commands.json not found. Please install 'bear > 3.0' and rebuild Verilator")
if not have_clang_check():
    test.skip("No libclang installed")

# some of the files are only used in Verilation
# and are only in "include" folder
srcfiles = test.glob_some(test.root + "/include/*.cpp")
srcfiles_str = " ".join(srcfiles)
clang_args = "-I" + test.root + "/include/ -I" + test.root + "/include/vltstd/ -fcoroutines-ts"

test.run(logfile=test.run_log_filename,
         tee=True,
         cmd=["python3", test.root + "/nodist/clang_check_attributes",
              "--verilator-root=" + test.root,
              "--cxxflags='" + clang_args + "'",
              srcfiles_str])  # yapf:disable

test.file_grep(test.run_log_filename, r'Number of functions reported unsafe: +(\d+)', 0)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

# This works with other vlt_alls, we we don't run it for them. It should
# fail with Verilator if --x-initial-edge is not specified.

import vltest_bootstrap

test.scenarios('vlt_all')
test.top_filename = "t/t_initial_edge.v"

test.compile()

test.execute(fails=True)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vltmt')

if not os.path.exists(test.root + "/.git"):
    test.skip("Not in a git repository")

test.compile(
    # Can't use --coverage and --savable together, so cheat and compile inline
    verilator_flags2=[
        "--cc", "--coverage-toggle --coverage-line --coverage-user", "--trace-vcd --vpi ",
        ("--timing" if test.have_coroutines else "--no-timing -Wno-STMTDLY"), "--prof-exec",
        "--prof-pgo", test.root + "/include/verilated_save.cpp"
    ],
    threads=2)

test.execute(
    all_run_flags=[" +verilator+prof+exec+file+/dev/null", " +verilator+prof+vlt+file+/dev/null"])

hit = {}
for filename in (test.glob_some(test.root + "/include/*.cpp") +
                 test.glob_some(test.root + "/include/*.h")):
    filename = os.path.basename(filename)
    if test.verbose:
        print("NEED: " + filename)
    hit[filename] = False

for dfile in test.glob_some(test.obj_dir + "/*.d"):
    wholefile = test.file_contents(dfile)
    for filename in wholefile.split():
        filename = os.path.basename(filename)
        if test.verbose:
            print("USED: " + filename)
        hit[filename] = True

for filename in sorted(hit.keys()):
    if (not hit[filename] and not re.search(r'_sc', filename) and not re.search(r'_fst', filename)
            and not re.search(r'_saif', filename) and not re.search(r'_thread', filename)
            and (not re.search(r'_timing', filename) or test.have_coroutines)):
        test.error("Include file not covered by t_verilated_all test: ", filename)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')
test.top_filename = "t/t_cover_lib.v"

test.compile(v_flags2=["--coverage t/t_cover_lib_c.cpp"],
             verilator_flags2=["--exe -Wall -Wno-DECLFILENAME"],
             make_flags=['CPPFLAGS_ADD=-DTEST_OBJ_DIR="' + test.obj_dir + '"'],
             make_top_shell=False,
             make_main=False)

test.execute()
test.files_identical_sorted(test.obj_dir + "/coverage1.dat", "t/t_cover_lib__1.out")
test.files_identical_sorted(test.obj_dir + "/coverage2.dat", "t/t_cover_lib__2.out")
test.files_identical_sorted(test.obj_dir + "/coverage3.dat", "t/t_cover_lib__3.out")
test.files_identical_sorted(test.obj_dir + "/coverage4.dat", "t/t_cover_lib__4.out")

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('simulator')

if test.nc:
    # For NC, compile twice, first just to generate DPI headers
    test.compile(nc_flags2=[
        "+ncdpiheader+" + test.obj_dir + "/dpi-exp.h", "+ncdpiimpheader+" + test.obj_dir +
        "/dpi-imp.h"
    ])

test.compile(
    v_flags2=["t/" + test.name + ".cpp"],
    # --no-decoration so .out file doesn't comment on source lines
    verilator_flags2=["-Wall -Wno-DECLFILENAME --no-decoration"],
    # NC: Gdd the obj_dir to the C include path
    nc_flags2=["+ncscargs+-I" + test.obj_dir],
    # ModelSim: Generate DPI header, add obj_dir to the C include path
    ms_flags2=["-dpiheader " + test.obj_dir + "/dpi.h", "-ccflags -I" + test.obj_dir])

if test.vlt_all:
    test.files_identical(test.obj_dir + "/" + test.vm_prefix + "__Dpi.h",
                         "t/" + test.name + "__Dpi.out")

test.execute(expect_filename=test.golden_filename)

test.passes()



# pylint: disable=R0914
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2026 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import os
import re


def run(test, *, verilator_flags2=()):
    variant, fmt = test.parse_name(r"t_trace_lib_(default|notop)_([a-z]+)")

    # All test use the same SV file
    test.top_filename = "t/t_trace_lib.v"
    # Any variations after the format name must yield the exact same trace
    test.golden_filename = test.py_filename.rpartition(fmt)[0] + fmt + ".out"

    verilator_common_flags = [
        "--cc",
        f"--trace-{fmt}",
        "--trace-underscore",  # Should not trace __Vhandle
        "--trace-max-width",
        "0",
        "--trace-max-array",
        "0",
        "--trace-structs"
    ]

    main_top_name = "top"
    match variant:
        case "default":
            pass
        case "notop":
            main_top_name = ""
        case _:
            test.error(f"Unhandled test variant '{variant}'")

    verilator_common_flags.extend(verilator_flags2)

    # Compile sub0 --lib-create
    test.vm_prefix = "Vsub0"
    test.compile(make_main=False,
                 threads=1,
                 verilator_make_gmake=False,
                 verilator_flags2=verilator_common_flags + [
                     "+define+SUB0",
                     "--lib-create",
                     "sub0",
                     "--top-module",
                     "sub0",
                 ])
    test.run(logfile=test.obj_dir + "/Vsub0_make.log",
             cmd=[os.environ["MAKE"], "-C", test.obj_dir, "-f", "Vsub0.mk", "libsub0.a"])
    # Compile sub1 --lib-create
    test.vm_prefix = "Vsub1"
    test.compile(make_main=False,
                 threads=1,
                 verilator_make_gmake=False,
                 verilator_flags2=verilator_common_flags + [
                     "+define+SUB1",
                     "--lib-create",
                     "sub1",
                     "--top-module",
                     "sub1",
                 ])
    test.run(logfile=test.obj_dir + "/Vsub1_make.log",
             cmd=[os.environ["MAKE"], "-C", test.obj_dir, "-f", "Vsub1.mk", "libsub1.a"])
    # Compile top with libraries
    test.vm_prefix = "Vlibs"
    test.compile(main_top_name=main_top_name,
                 verilator_flags2=verilator_common_flags + [
                     "+define+TOP", test.obj_dir + "/sub0.sv", "libsub0.a",
                     test.obj_dir + "/sub1.sv", "libsub1.a"
                 ])

    # Compile simply without libraries
    test.vm_prefix = "Vnonl"
    test.main_filename = test.obj_dir + "/Vnonl__main.cpp"
    test.compile(main_top_name=main_top_name,
                 verilator_flags2=verilator_common_flags +
                 ["+define+TOP", "+define+SUB0", "+define+SUB1"])

    trace_libs = test.trace_filename.replace("simx", "libs")
    trace_nonl = test.trace_filename.replace("simx", "nonl")

    # Run the --lib-create model
    test.execute(executable=test.obj_dir + "/Vlibs")
    test.run(cmd=["mv", test.trace_filename, trace_libs])
    # Run the simple model without libraries
    test.execute(executable=test.obj_dir + "/Vnonl")
    test.run(cmd=["mv", test.trace_filename, trace_nonl])

    # Scope structure must match exactly, check only in vcd
    if fmt == "vcd":
        with open(trace_nonl, 'r', encoding='utf8') as fnonl, \
             open(trace_libs, 'r', encoding='utf8') as flibs:
            for la, lb in zip(fnonl, flibs):
                la = re.sub(r'^(\s*\$var\s+\S+\s+\S+\s+)\S+(.*)$', r'\1CODE\2', la)
                lb = re.sub(r'^(\s*\$var\s+\S+\s+\S+\s+)\S+(.*)$', r'\1CODE\2', lb)
                if la != lb:
                    test.error_keep_going("VCD header mismatch: '{}' !~ '{}'".format(
                        la.strip(), lb.strip()))
                if "enddefinitions" in la:
                    break

    test.trace_identical(trace_libs, trace_nonl)
    # The --lib-create must match the reference
    test.trace_identical(trace_libs, test.golden_filename)

    test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('simulator')

test.compile()

if test.vlt_all:
    # The word 'this' (but only the whole word 'this' should have been replaced
    # in the contents.
    has_this = False
    has_xthis = False
    has_thisx = False
    has_xthisx = False
    for filename in test.glob_some(test.obj_dir + "/" + test.vm_prefix + "___024root__0.cpp"):
        text = test.file_contents(filename)
        if re.search(r'\bthis->clk\b', text):
            has_this = True
        if re.search(r'\bxthis\b', text):
            has_xthis = True
        if re.search(r'\bthisx\b', text):
            has_thisx = True
        if re.search(r'\bxthisx\b', text):
            has_xthisx = True

    if has_this:
        test.error("Some file has 'this->clk'")
    if not has_xthis:
        test.error("No file has 'xthis'")
    if not has_thisx:
        test.error("No file has 'thisx'")
    if not has_xthisx:
        test.error("No file has 'xthisx'")

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2025 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt_all')

lib_dir = test.obj_dir + "/sub"
test.mkdir_ok(lib_dir)

test.run(logfile=lib_dir + "/verilator.log",
         cmd=[
             "perl", os.environ["VERILATOR_ROOT"] + "/bin/verilator", "-cc", "-Mdir", lib_dir,
             "--lib-create", "sub", "--prefix", "Vsub", "+define+LIB_CREATE", test.top_filename
         ],
         verilator_run=True)

test.run(logfile=lib_dir + "/make.log", cmd=[os.environ["MAKE"], "-C", lib_dir, "-f", "Vsub.mk"])

test.compile(verilator_flags2=["--binary", "-LDFLAGS", "sub/libsub.a", lib_dir + "/sub.sv"])

test.execute(xsim_run_flags2=["--sv_lib", lib_dir + "/libsecret", "--dpi_absolute"])

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt_all')
test.sim_time = 2000000
test.top_filename = "t/t_dfg_peephole.v"

if not os.path.exists(test.root + "/.git"):
    test.skip("Not in a git repository")

# Read optimizations
optimizations = []

hdrFile = "../src/V3DfgPeepholePatterns.h"
with open(hdrFile, 'r', encoding="utf8") as hdrFh:
    prevOpt = ""
    lineno = 0
    for line in hdrFh:
        lineno += 1
        m = re.search(r'^\s*_FOR_EACH_DFG_PEEPHOLE_OPTIMIZATION_APPLY\(macro, (\w+)\)', line)
        if not m:
            continue
        opt = m.group(1)
        if prevOpt > opt:
            test.error(hdrFile + ":" + str(lineno) + ": '" + opt + "; is not in sorted order")
        prevOpt = opt
        # Trivial, but also hard to trigger on purpose as it depends on rewrite order
        if opt == "REPLACE_WITH_EQUIVALENT":
            continue
        optimizations.append(opt)

if len(optimizations) < 1:
    test.error("no optimizations defined in " + hdrFile)

# Generate the equivalence checks and declaration boilerplate
rdFile = test.top_filename
plistFile = test.obj_dir + "/portlist.vh"
pdeclFile = test.obj_dir + "/portdecl.vh"
checkFile = test.obj_dir + "/checks.h"
with open(rdFile, 'r', encoding="utf8") as rdFh, \
     open(plistFile, 'w', encoding="utf8") as plistFh, \
     open(pdeclFile, 'w', encoding="utf8") as pdeclFh, \
     open(checkFile, 'w', encoding="utf8") as checkFh:
    for line in rdFh:
        m = re.search(r'^\s*.*`signal\((\w+),', line)
        if not m:
            continue
        sig = m.group(1)
        plistFh.write(sig + ",\n")
        pdeclFh.write("output " + sig + ";\n")
        checkFh.write("if (ref." + sig + " != opt." + sig + ") {\n")
        checkFh.write("    std::cout << \"Mismatched " + sig + "\" << std::endl;\n")
        checkFh.write("    std::cout << \"Ref: \" << VL_TO_STRING(ref." + sig +
                      ") << std::endl;\n")
        checkFh.write("    std::cout << \"Opt: \" << VL_TO_STRING(opt." + sig +
                      ") << std::endl;\n")
        checkFh.write("    std::exit(1);\n")
        checkFh.write("}\n")

# Compile un-optimized
test.compile(verilator_flags2=[
    "--stats",
    "--build",
    "-fno-dfg",
    "+incdir+" + test.obj_dir,
    "-Mdir", test.obj_dir + "/obj_ref",
    "--prefix", "Vref"
])  # yapf:disable

# Compile optimized - also builds executable

extraArgs = []
if test.name == "t_dfg_peephole_off_all":
    extraArgs.append("-fno-dfg-peephole")
if test.name == "t_dfg_peephole_off_each":
    for opt in optimizations:
        opt = opt.lower()
        opt = re.sub(r"_", "-", opt)
        extraArgs.append("-fno-dfg-peephole-" + opt)

test.compile(verilator_flags2=[
    "--stats",
    "--build",
    "--exe",
    "+incdir+" + test.obj_dir,
    "-Mdir", test.obj_dir + "/obj_opt",
    "--prefix", "Vopt",
    "-fno-const-before-dfg",  # Otherwise V3Const makes testing painful
    "-fno-lift-expr", # Assumes V3Const run prior to V3LiftExpr
    "-fdfg-synthesize-all",
    "--dump-dfg",  # To fill code coverage
    "-CFLAGS \"-I .. -I ../obj_ref\"",
    "../obj_ref/Vref__ALL.a",
    "../../t/t_dfg_peephole.cpp"
] + extraArgs)  # yapf:disable


def check(name, enabled):
    name = name.lower()
    name = re.sub(r'_', ' ', name)
    pattern = r'DFG, Peephole, ' + name + r'\s+([1-9]\d*)\s*$'
    if enabled:
        test.file_grep(test.obj_dir + "/obj_opt/Vopt__stats.txt", pattern)
    else:
        test.file_grep_not(test.obj_dir + "/obj_opt/Vopt__stats.txt", pattern)


# Check all optimizations defined in
for opt in optimizations:
    check(opt, test.name == "t_dfg_peephole")

test.file_grep_not(test.obj_dir + "/obj_opt/Vopt__stats.txt",
                   r'DFG.*non-representable.*\s[1-9]\d*$')

# Execute test to check equivalence
test.execute(executable=test.obj_dir + "/obj_opt/Vopt")

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2025 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('dist')

test.passes()

test.timeout(1)

# Check whether the soft limit is set to 1s
test.run(cmd=["bash", "-c", "\"[ '$(ulimit -St)' -eq 1 ] || exit 1\""])

# Set the hard limit to 2s (in case soft limit fails) and run a command which spins until signalled
test.run(cmd=["bash", "-c", "\"trap 'exit 0' SIGXCPU; ulimit -Ht 2; while :; do :; done\""])

test.passes()



# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2026 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0


def run(test, *, verilator_flags2=()):
    fmt, = test.parse_name(r"t_trace_hier_([a-z]+)")

    # All test use the same SV file
    test.top_filename = "t/t_trace_hier.v"
    # Any variations after the format name must yield the exact same trace
    test.golden_filename = test.py_filename.rpartition(fmt)[0] + fmt + ".out"

    flags = [
        f"--trace-{fmt}", "-j", "4", "t/t_trace_hier_sub/t_trace_hier.vlt", "--top-module", "t",
        "--hierarchical", "-F", "t/t_trace_hier_sub/top.vc"
    ]
    flags.extend(verilator_flags2)

    # Run test
    test.compile(verilator_flags2=flags)

    test.execute()

    test.trace_identical(test.trace_filename, test.golden_filename)

    test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.priority(30)
test.scenarios('vlt_all')

# stats will be deleted but generation will be skipped if libs of hierarchical blocks exist.
test.clean_objs()

# CI environment offers 2 VCPUs, 2 thread setting causes the following warning.
# %Warning-UNOPTTHREADS: Thread scheduler is unable to provide requested parallelism; consider asking for fewer threads.
# So use 6 threads here though it's not optimal in performance, but ok.

test.compile(
    v_flags2=['t/t_hier_block.cpp'],
    verilator_flags2=[
        '--stats',
        '--hierarchical',
        '--Wno-TIMESCALEMOD',  #
        '-GPARAM_A=100',
        '-pvalue+PARAM_B=200',
        '-DPARAM_OVERRIDE',  #
        '--CFLAGS',
        '"-O0 -pipe -DCPP_MACRO=cplusplus"'
    ],
    threads=(6 if test.vltmt else 1))

test.execute()

test.file_grep(test.obj_dir + "/Vsub0/sub0.sv", r'^\s+\/\/\s+timeprecision\s+(\d+)ps;', 1)
test.file_grep(test.obj_dir + "/Vsub0/sub0.sv", r'^module\s+(\S+)\s+', "sub0")
test.file_grep(test.obj_dir + "/Vsub1/sub1.sv", r'^module\s+(\S+)\s+', "sub1")
test.file_grep(test.obj_dir + "/Vsub2/sub2.sv", r'^module\s+(\S+)\s+', "sub2")
test.file_grep(test.stats, r'HierBlock,\s+Hierarchical blocks\s+(\d+)', 14)
test.file_grep(test.run_log_filename, r'MACRO:(\S+) is defined', "cplusplus")

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('dist')

for basename in [
        "t_vlcov_data_a.dat", "t_vlcov_data_b.dat", "t_vlcov_data_c.dat", "t_vlcov_data_d.dat"
]:
    test.run(cmd=[
        os.environ["VERILATOR_ROOT"] + "/bin/verilator_coverage", "t/" + basename, "--debug",
        "--debugi 9"
    ],
             tee=test.verbose,
             verilator_run=True)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')

if test.have_coroutines:
    test.run(cmd=[os.environ["VERILATOR_ROOT"] + "/bin/verilator --get-supported COROUTINES"],
             logfile=test.obj_dir + "/vlt_coroutines.log",
             expect_filename="t/t_flag_supported_1.out",
             verilator_run=True)

if test.have_sc:
    test.run(cmd=[os.environ["VERILATOR_ROOT"] + "/bin/verilator --get-supported SYSTEMC"],
             logfile=test.obj_dir + "/vlt_systemc.log",
             expect_filename="t/t_flag_supported_1.out",
             verilator_run=True)

test.run(cmd=[os.environ["VERILATOR_ROOT"] + "/bin/verilator --get-supported DOES_NOT_EXIST"],
         logfile=test.obj_dir + "/vlt_does_not_exist.log",
         expect_filename="t/t_flag_supported_empty.out",
         verilator_run=True)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vltmt')

# Use a top file which we are sure to be parallelizable
test.top_filename = "t/t_gen_alw.v"

test.compile(v_flags2=["--dumpi-graph 6"], threads=2)

for dotname in [
        "linkcells", "task_call", "gate_graph", "gate_final", "acyc_simp", "orderg_pre",
        "orderg_acyc", "orderg_order", "orderg_domain", "ordermv_initial", "ordermv_hazards",
        "ordermv_contraction", "ordermv_transitive1", "orderg_done", "pack", "schedule"
]:
    # Some files with identical prefix are generated multiple times during
    # Verilation. Ensure that at least one of each dotname-prefixed file is generated.
    dotFiles = test.glob_some(test.obj_dir + "/*" + dotname + ".dot")
    for dotFilename in dotFiles:
        test.file_grep(dotFilename, r'digraph v3graph')

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# Copyright 2024 by Wilson Snyder. This program is free software; you
# can redistribute it and/or modify it under the terms of either the GNU
# Lesser General Public License Version 3 or the Perl Artistic License
# Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')

test.top_filename = "t/t_covergroup_cross.v"

test.compile(verilator_flags2=['--coverage', '--Wno-COVERIGN'])

test.execute()

test.run(cmd=[
    os.environ["VERILATOR_ROOT"] + "/bin/verilator_coverage",
    "--annotate",
    test.obj_dir + "/annotated",
    "--annotate-points",
    test.obj_dir + "/coverage.dat",
],
         verilator_run=True)

test.files_identical(test.obj_dir + "/annotated/t_covergroup_cross.v",
                     "t/" + test.name + ".annotate.out")

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt', 'xsim')

if test.benchmark:
    test.sim_time = test.benchmark * 100

secret_prefix = "secret"
secret_dir = test.obj_dir + "/" + secret_prefix
test.mkdir_ok(secret_dir)
trace_opt = ""

# Always compile the secret file with Verilator no matter what simulator
#   we are testing with
test.run(logfile=secret_dir + "/vlt_compile.log",
         cmd=["perl", os.environ["VERILATOR_ROOT"] + "/bin/verilator",
              '--no-timing',
              trace_opt,
              "--prefix", "Vt_lib_prot_secret",
              "-cc",
              "-Mdir", secret_dir,
              "--protect-lib", secret_prefix,
              "--protect-key", "secret-key",
              "t/t_lib_prot_secret.v"],
         verilator_run=True)  # yapf:disable

test.run(logfile=secret_dir + "/secret_gcc.log",
         cmd=[os.environ["MAKE"], "-C", secret_dir, "-f", "Vt_lib_prot_secret.mk"])

test.compile(verilator_flags2=['--no-timing',
                               trace_opt,
                               "-LDFLAGS", secret_prefix + "/libsecret.a",
                               secret_dir + "/secret.sv"],
             xsim_flags2=[secret_dir + "/secret.sv"])  # yapf:disable

test.execute(xsim_run_flags2=["--sv_lib", secret_dir + "/libsecret", "--dpi_absolute"])

if test.vlt and test.trace:
    # We can see the ports of the secret module
    test.file_grep(test.trace_filename, r'accum_in')
    # but we can't see what's inside
    test.file_grep_not(test.trace_filename, r'secret_')

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

# Test for bin/verilator_gantt,

import vltest_bootstrap
import sys

test.scenarios('vltmt')
test.top_filename = "t/t_gantt.v"
test.pli_filename = "t/t_gantt_c.cpp"

test.compile(
    verilator_flags2=["--prof-exec", test.pli_filename],
    # Checks below care about thread count
    threads=4)

# We need several experiments to make sure that the algorithm is working
trials = 4
for trial in range(0, trials):
    print("--------- Trial %d" % trial)

    test.execute(  # Test fail: run_env='numactl -m 0 -C 0,0,0,0',
        all_run_flags=[
            "+verilator+prof+exec+start+2", " +verilator+prof+exec+window+2",
            " +verilator+prof+exec+file+" + test.obj_dir + "/profile_exec.dat"
        ])

    gantt_log = test.obj_dir + "/gantt.log"

    test.run(cmd=[
        os.environ["VERILATOR_ROOT"] + "/bin/verilator_gantt", "--no-vcd", test.obj_dir +
        "/profile_exec.dat", "| tee " + gantt_log
    ])

    if sys.platform != "darwin":
        test.file_grep(gantt_log, r'CPU info:')
        test.file_grep(gantt_log, r'NUMA status += (assigned|%Warning: no /proc/cpuinfo)')
        # False fails occasionally
        # test.file_grep_not(gantt_log, r'%Warning:')  # e.g. There were fewer CPUs (1) than threads (3).

if sys.platform != "darwin":
    # Test disabling NUMA assignment
    gantt_log_numa_none = test.obj_dir + "/gantt_numa_none.log"
    test.execute(run_env='VERILATOR_NUMA_STRATEGY=none',
                 all_run_flags=[
                     "+verilator+prof+exec+start+2", " +verilator+prof+exec+window+2",
                     " +verilator+prof+exec+file+" + test.obj_dir + "/profile_exec.dat"
                 ])
    test.run(cmd=[
        os.environ["VERILATOR_ROOT"] + "/bin/verilator_gantt", "--no-vcd", test.obj_dir +
        "/profile_exec.dat", "| tee " + gantt_log_numa_none
    ])
    test.file_grep(gantt_log_numa_none, r'NUMA status += no NUMA assignment requested')

    # Test invalid NUMA assignment
    gantt_log_numa_invalid = test.obj_dir + "/gantt_numa_invalid.log"
    test.execute(run_env='VERILATOR_NUMA_STRATEGY=invalid_value',
                 all_run_flags=[
                     "+verilator+prof+exec+start+2", " +verilator+prof+exec+window+2",
                     " +verilator+prof+exec+file+" + test.obj_dir + "/profile_exec.dat"
                 ])
    test.run(cmd=[
        os.environ["VERILATOR_ROOT"] + "/bin/verilator_gantt", "--no-vcd", test.obj_dir +
        "/profile_exec.dat", "| tee " + gantt_log_numa_invalid
    ])
    # %Warning: unknown VERILATOR_NUMA_STRATEGY value 'invalid_value'
    test.file_grep(
        gantt_log_numa_invalid,
        r"NUMA status += %Warning: unknown VERILATOR_NUMA_STRATEGY value 'invalid_value'")

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2025 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')


def fixup(path):
    lines = []
    removeBeginKeywords = True
    with open(path, "r", encoding="utf-8") as rfd:
        for line in rfd:
            # Remove "all arguments" line, which can differ
            if line.startswith("//   Arguments:"):
                lines.append("//   Arguments: <REDACTED>")
            # Remove first `begin_keywords, which will be re-inserted on reruns
            elif removeBeginKeywords and line.startswith("`begin_keywords"):
                removeBeginKeywords = False
                lines.append("// " + line)
            else:
                lines.append(line)
    with open(path, "w", encoding="utf-8") as wfd:
        for line in lines:
            wfd.write(line)


obj_dir_1 = test.obj_dir + "/obj_dir_1"
test.mkdir_ok(obj_dir_1)
dump_1 = obj_dir_1 + "/Vprefix__inputs.vpp"
test.run(
    logfile=obj_dir_1 + "/vlt_compile.log",
    cmd=[
        "perl",
        os.environ["VERILATOR_ROOT"] + "/bin/verilator",
        "-cc",
        "-Mdir",
        obj_dir_1,
        "--prefix",
        "Vprefix",
        "--top-module",
        "lpm_divide",
        "--no-timing",
        "-CFLAGS",
        "'-O3 --foo'",  # Argument with whitespace
        "-CFLAGS",
        "'-DDQUOTE=\"'",  # Argument with quote
        "--dump-inputs",
        "t/t_altera_lpm.v",
        "t/t_math_cond_huge.v"
    ])
fixup(dump_1)

obj_dir_2 = test.obj_dir + "/obj_dir_2"
test.mkdir_ok(obj_dir_2)
dump_2 = obj_dir_2 + "/Vprefix__inputs.vpp"
test.run(
    logfile=obj_dir_2 + "/vlt_compile.log",
    cmd=[
        "perl",
        os.environ["VERILATOR_ROOT"] + "/bin/verilator",
        "-Mdir",
        obj_dir_2,
        "-f",
        dump_1,
        dump_1,
        "--debug",  # --debug also dumps the same way
        "--debugi",
        "1"
    ])
fixup(dump_2)

obj_dir_3 = test.obj_dir + "/obj_dir_3"
test.mkdir_ok(obj_dir_3)
dump_3 = obj_dir_3 + "/Vprefix__inputs.vpp"
test.run(logfile=obj_dir_3 + "/vlt_compile.log",
         cmd=[
             "perl",
             os.environ["VERILATOR_ROOT"] + "/bin/verilator",
             "-Mdir",
             obj_dir_3,
             "-f",
             dump_2,
             dump_2,
             "--dump-inputs",
         ])
fixup(dump_3)

test.files_identical(dump_1, dump_2)
test.files_identical(dump_1, dump_3)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('dist')

if not os.path.exists(test.root + "/.git"):
    test.skip("Not in a git repository")

### Must trim output before and after our file list
cmd = "cd " + test.root + " && git ls-files --exclude-standard"
files = test.run_capture(cmd)
if test.verbose:
    print("ST " + files)
names = {}
for filename in files.split():
    if "include/vltstd/vpi_user.h" in filename:  # IEEE Standard file - can't change it
        continue
    if "include/gtkwave/" in filename:  # Standard file - can't change it
        continue
    if "test_regress/t/uvm/" in filename:  # Standard file - can't change it
        continue
    filename = os.path.join(test.root, filename)
    if not os.path.exists(filename):
        continue
    with open(filename, 'r', encoding='latin-1') as fh:
        for line in fh:
            if "include" not in line:
                continue
            line = line.rstrip()
            hit = (re.search(r'\bassert\.h', line) or re.search(r'\bctype\.h', line)
                   or re.search(r'\berrno\.h', line) or re.search(r'\bfloat\.h', line)
                   or re.search(r'\blimits\.h', line) or re.search(r'\blocale\.h', line)
                   or re.search(r'\bmath\.h', line) or re.search(r'\bsetjmp\.h', line)
                   or re.search(r'\bsignal\.h', line) or re.search(r'\bstdarg\.h', line)
                   or re.search(r'\bstdbool\.h', line) or re.search(r'\bstddef\.h', line)
                   or re.search(r'\bstdio\.h', line) or re.search(r'\bstdlib\.h', line)
                   or re.search(r'\bstring\.h', line)
                   or (re.search(r'\btime\.h', line) and not re.search(r'sys/time.h', line)))
            #Not yet: r'\bstdint\.h'
            if not hit:
                continue
            names[filename + ": " + line] = True

if len(names):
    test.error("Files like stdint.h instead of cstdint:\n    " +
               "\n    ".join(sorted(names.keys())))

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt_all')

test.top_filename = "t/t_flag_csplit.v"


def check_splits():
    got1 = False
    gotSyms1 = False
    for filename in test.glob_some(test.obj_dir + "/*.cpp"):
        if re.search(r'Syms__.*__1', filename):
            gotSyms1 = True
        elif re.search(r'__1', filename):
            got1 = True
    if not got1:
        test.error("No __1 split file found")
    if not gotSyms1:
        test.error("No Syms__*__1 split file found")


def check_no_all_file():
    for filename in test.glob_some(test.obj_dir + "/*.cpp"):
        if re.search(r'__ALL.cpp', filename):
            test.error("__ALL.cpp file found: " + filename)


def check_cpp(filename):
    size = os.path.getsize(filename)
    if test.verbose:
        print("  File %6d  %s\n" % (size, filename))
    funcs = []
    with open(filename, 'r', encoding="utf8") as fh:
        for line in fh:
            m = re.search(r'^(void|IData)\s+(.*::.*){', line)
            if not m:
                continue
            func = m.group(2)
            func = re.sub(r'\(.*$', '', func)
            if test.verbose:
                print("\tFunc " + func)
            if (re.search(r'(::_eval_initial_loop$', func) or re.search(r'::__Vconfigure$', func)
                    or re.search(r'::trace$', func) or re.search(r'::traceInit$', func)
                    or re.search(r'::traceFull$', func) or re.search(r'::final$', func)
                    or re.search(r'::prepareClone$', func) or re.search(r'::atClone$', func)):
                continue
            funcs.append(func)

    if len(funcs) > 0:
        test.error("Split had multiple functions in $filename\n\t" + "\n\t".join(funcs))


def check_gcc_flags(filename):
    with open(filename, 'r', encoding="utf8") as fh:
        for line in fh:
            line = line.rstrip()
            if test.verbose:
                print(":log: " + line)
            if re.search(r'' + test.vm_prefix + r'\S*\.cpp', line):
                filetype = "slow" if re.search(r'(Slow|Syms)', line) else "fast"
                opt = "fast" if re.search(r'-O2', line) else "slow"
                if test.verbose:
                    print(filetype + ", " + opt + ", " + line)
                if filetype != opt:
                    test.error(filetype + " file compiled as if was " + opt + ": " + line)
            elif re.search(r'.cpp', line) and not re.search(r'-Os', line):
                test.error("library file not compiled with OPT_GLOBAL: " + line)


# This rule requires GNU make > 4.1 (or so, known broken in 3.81)
#%__Slow.o: %__Slow.cpp
#        $(OBJCACHE) $(CXX) $(CXXFLAGS) $(CPPFLAGS) $(OPT_SLOW) -c -o $@ $<
if not test.make_version or float(test.make_version) < 4.1:
    test.skip("Test requires GNU Make version >= 4.1")

test.compile(v_flags2=["--exe",
                       "--trace-vcd",
                       "--output-split 1",
                       "--output-groups 2",
                       "--output-split-cfuncs 1",
                       "--stats",
                       "--dumpi-V3EmitMk 9",  # Dev coverage of the V3EmitMk debug printer
                       "../" + test.main_filename],
             verilator_make_gmake=False)  # yapf:disable

# We don't use the standard test_regress rules, as want to test the rules
# properly build
test.run(logfile=test.obj_dir + "/vlt_gcc.log",
         tee=test.verbose,
         cmd=[os.environ["MAKE"],
              "-C " + test.obj_dir,
              "-f "+test.vm_prefix+".mk",
              "-j 4",
              "VM_PREFIX="+test.vm_prefix,
              "TEST_OBJ_DIR="+test.obj_dir,
              "CPPFLAGS_DRIVER=-D"+test.name.upper(),
              ("CPPFLAGS_DRIVER2=-DTEST_VERBOSE=1" if test.verbose else ""),
              "OPT_FAST=-O2",
              "OPT_SLOW=-O0",
              "OPT_GLOBAL=-Os",
              ])  # yapf:disable

test.execute()

# Splitting should set VM_PARALLEL_BUILDS to 1 by default
test.file_grep(test.obj_dir + "/" + test.vm_prefix + "_classes.mk", r'VM_PARALLEL_BUILDS\s*=\s*1')
check_splits()
check_no_all_file()
check_gcc_flags(test.obj_dir + "/vlt_gcc.log")

# Check that only vm_classes_*.cpp are to be compiled
test.file_grep_not(test.obj_dir + "/" + test.vm_prefix + "_classes.mk", "sub")
test.file_grep(test.obj_dir + "/" + test.vm_prefix + "_classes.mk", "vm_classes_Slow_1")
test.file_grep(test.obj_dir + "/" + test.vm_prefix + "_classes.mk", "vm_classes_1")
test.file_grep_not(test.obj_dir + "/" + test.vm_prefix + "_classes.mk", "vm_classes_Slow_2")
test.file_grep_not(test.obj_dir + "/" + test.vm_prefix + "_classes.mk", "vm_classes_2")

# Check combine count
nFiles = int(test.file_grep(test.stats, r'Node count, CFILE + (\d+)')[0])
if nFiles < 200:
    test.error("Expecting >= 200 files, but got " + str(nFiles))
test.file_grep(test.stats, r'Makefile targets, VM_CLASSES_FAST + (\d+)', 2)
test.file_grep(test.stats, r'Makefile targets, VM_CLASSES_SLOW + (\d+)', 2)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('simulator')

if not test.have_solver:
    test.skip("No constraint solver installed")

test.compile(
    # Ensure we test captures of static variables
    verilator_flags2=["--fno-inline"])

test.execute()

for filename in test.glob_some(test.obj_dir + "/" + test.vm_prefix + "*Baz*.cpp"):
    # Check that "Baz" has no constrained random generator
    test.file_grep_not(filename, "this->__PVT__constraint")

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2026 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('simulator')

# --output-split triggers PCH generation (.gch files) for parallel builds
test.compile(verilator_flags2=["--output-split 1", "--output-split-cfuncs 1"])

# Verify PCH header was generated
test.glob_some(test.obj_dir + "/*__pch.h")

# Verify split files were produced
test.glob_some(test.obj_dir + "/*__1.cpp")

# Verify parallel builds enabled
test.file_grep(test.obj_dir + "/" + test.vm_prefix + "_classes.mk", r'VM_PARALLEL_BUILDS\s*=\s*1')

test.execute()

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('dist')

Contributors = {'github action': True}
Authors = {}


def read_contributors(filename):
    with open(filename, 'r', encoding="utf8") as fh:
        # Assumes git .mailmap format
        for line in fh:
            line = line.rstrip()
            line = re.sub(r' *<[^>]*>', '', line)
            Contributors[line] = True


def read_user():
    cmd = "cd " + test.root + " && git diff-index --quiet HEAD --"
    changes = test.run_capture(cmd, check=False)
    changes = changes.rstrip()
    if changes == "":
        if test.verbose:
            print("No git changes")
    else:
        # Uncommitted changes, so check the user's git name
        user = test.run_capture("git config user.name")
        user = user.rstrip()
        if user:
            Authors[user] = True


def read_authors():
    # Check recent commits in case did commit
    cmd = "git log '--pretty=format:%aN <%aE>' | head -5"
    git_auths = test.run_capture(cmd)
    for line in git_auths.splitlines():
        line = re.sub(r' *<[^>]*>', '', line)
        Authors[line] = True


def check():
    read_contributors(test.root + "/docs/CONTRIBUTORS")
    read_user()
    read_authors()
    for author in sorted(Authors.keys()):
        if test.verbose:
            print("Check: " + author)
        if re.search(r'\[bot\]', author):
            continue
        if author in Contributors:
            continue
        # Slower subset-of-string test
        ok = False
        for contrib2 in sorted(Contributors.keys()):
            if author in contrib2:
                ok = True
                break
        if ok:
            continue

        test.error("Certify your contribution by sorted-inserting '" + author +
                   "' into docs/CONTRIBUTORS.\n"
                   "   If '" + author +
                   "' is not your real name, please fix 'name=' in ~/.gitconfig\n"
                   "   Also check your https://github.com account's Settings->Profile->Name\n"
                   "   matches your ~/.gitconfig 'name='.\n")


if 'VERILATOR_TEST_NO_CONTRIBUTORS' in os.environ:
    test.skip("Skipping due to VERILATOR_TEST_NO_CONTRIBUTORS")
if not os.path.exists(test.root + "/.git"):
    test.skip("Not in a git repository")

check()
test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')

opts = []
# Typo
opts += ["-ccc"]
# Typo of an option that starts with "--"
opts += ["--ccc"]
# Typo of an option that starts with "-no-"
opts += ["-no-asserT"]
# Typo of an option that starts with "-no"
opts += ["-noasserT"]
# Typo of an option that allows "-no"
opts += ["-asserT"]
# Typo of an option that starts with '+'
opts += ["+definE+A=B"]
# Typo that takes arg
opts += ["-CFLAGs -ggdb"]
# Typo of an undocumented option
opts += ["-debug-aborT"]
# Typo of "-Wno" for partial match
opts += ["-Won-SPLITVAR"]
# Typo after -Wno- for partial match
opts += ["-Wno-SPLITVER"]
# Typo of -language
opts += ["-language 1364-1997"]

cmd = ""
for var in opts:
    cmd += os.environ["VERILATOR_ROOT"] + "/bin/verilator " + var + "; "

test.run(cmd=[cmd],
         verilator_run=True,
         logfile=test.obj_dir + "/sim.log",
         fails=True,
         expect_filename=test.golden_filename)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: FSM lowered coverage declaration dump test
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2026 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

from pathlib import Path

import vltest_bootstrap

test.scenarios('vlt')
test.top_filename = "t/t_cover_fsm_policy_accept_multi.v"

# Dump the lowered AST so AstCoverOtherDecl::dump() sees FSM metadata-bearing
# coverage declarations directly. This avoids JSON/schema coupling while still
# covering the dump-side formatting for fv/ff/ft/fg.
test.lint(v_flags=["--coverage-fsm", "--dump-tree"])

tree_files = [Path(filename) for filename in test.glob_some(test.obj_dir + "/*.tree")]
tree_texts = [filename.read_text(encoding="utf8") for filename in tree_files]

assert any("COVEROTHERDECL" in text and " fv=t.style_u.state" in text for text in tree_texts)
assert any(
    "COVEROTHERDECL" in text and " ff=ANY" in text and " ft=S0" in text and " fg=reset" in text
    for text in tree_texts)
assert any("COVEROTHERDECL" in text and " ff=default" in text and " ft=S0" in text
           and " fg=default" in text for text in tree_texts)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap
import datetime

test.scenarios('dist')

RELEASE_OK_RE = r'(^test_regress/t/.*\.(cpp|h|map|mk|sv|v|vlt)|^test_regress/t_done/|^examples/)'

EXEMPT_AUTHOR_RE = r'(^ci/|^nodist/fastcov.py|^nodist/fuzzer|^test_regress/t/.*\.(cpp|h|mk|s?vh?|vlt)$)'

EXEMPT_FILES_RE = r'(^\.|/\.|\.gitignore$|\.dat|\.gprof|\.mem|\.out$|\.png$|\.tree|\.vc$|\.vcd$|^\.)'

EXEMPT_FILES_LIST = """
    CITATION.cff
    CPPLINT.cfg
    LICENSE
    LICENSES/
    REUSE.toml
    ci/ci-win-compile.ps1
    ci/ci-win-test.ps1
    ci/docker/buildenv/wavetools.conf
    ci/codecov
    docs/CONTRIBUTORS
    docs/_static
    docs/gen
    docs/spelling.txt
    docs/verilated.dox
    include/fstcpp
    include/vltstd
    install-sh
    src/mkinstalldirs
    test_regress/t/t_altera_lpm.v
    test_regress/t/t_randsequence_svtests.v
    test_regress/t/uvm/
    verilator.pc.in
    """

Exempt_Files_List_Re = list(map(re.escape, EXEMPT_FILES_LIST.split()))
Exempt_Files_List_Re = '^(' + '|'.join(Exempt_Files_List_Re) + ")"
# pprint(Exempt_Files_List_Re)

if not os.path.exists(test.root + "/.git"):
    test.skip("Not in a git repository")

cmd = "cd " + test.root + " && git ls-files --exclude-standard"
out = test.run_capture(cmd)

year = datetime.datetime.now().year

files = {}
out = re.sub(r'\s+', ' ', out)
for filename in out.split():
    if re.search(EXEMPT_FILES_RE, filename):
        continue
    if re.search(Exempt_Files_List_Re, filename):
        continue
    files[filename] = True

for filename in files:
    open_filename = os.path.join(test.root, filename)
    if not os.path.exists(open_filename):
        continue

    if "test_regress/t" in filename:
        yeardash = str(year)
    else:
        yeardash = str(year) + '-' + str(year)

    with open(open_filename, 'r', encoding="utf8") as fh:
        spdx = None
        copyright_msg = None
        for line in fh:
            line = line.rstrip()
            if 'SP' + 'DX-License-Identifier:' in line:
                spdx = line
            elif re.search(r'(SP()DX-FileCopyrightText: 20[0-9][0-9])', line):
                copyright_msg = line
                if 'Wilson Snyder' in line:
                    pass
                elif re.search(EXEMPT_AUTHOR_RE, filename):
                    pass
                else:
                    print("   " + copyright_msg)
                    test.error_keep_going(filename + ": Please use standard 'SP" +
                                          "DX-FileCopyrightText: " + yeardash + " Wilson Snyder'")

        if not copyright_msg:
            test.error_keep_going(filename + ": Please add standard 'SP" +
                                  "DX-FileCopyrightText: " + yeardash +
                                  " ...', similar to in other files")
        if not spdx:
            test.error_keep_going(filename + ": Please add standard 'SP" +
                                  "DX-License-Identifier: ...', similar to in other files")

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2026 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')

# Reuse the same design.  '--x-assign 0' makes the auto-inserted out-of-range guard on a
# non-power-of-two bit-select a plain '(idx <= W-1) && vec[idx]' (AstLogAnd), rather than
# the ternary '(idx <= W-1) ? vec[idx] : <x>' (AstCond) produced under the driver's default
# '--x-assign unique'.  This exercises the matcher's other guard-peel branch.
test.top_filename = "t/t_bit_scan_loops.v"

test.compile(verilator_flags2=['--stats', '--unroll-count', '0', '--x-assign', '0'])

# Same lowering counts as the default run -- only the guard shape differs, not the result.
test.file_grep(test.stats,
               r'Optimizations, Loop unrolling, Lowered priority-encoder to mostsetbitp1\s+(\d+)',
               8)
test.file_grep(test.stats,
               r'Optimizations, Loop unrolling, Lowered count-set-bits to countones\s+(\d+)', 1)

test.execute()

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2026 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')

test.compile()

badfile = "/does-not-exist/badfile.log"
logfile = test.obj_dir + "/logfile.log"

log_flags = '+verilator+log+file+' + badfile
test.execute(all_run_flags=[log_flags, ">", logfile, "2>&1"], fails=True)

test.files_identical(logfile, test.golden_filename)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap
import collections

test.scenarios('dist')

Tabs_Exempt_Re = r'(\.out$)|(/fstcpp)|(Makefile)|(\.mk$)|(\.mk\.in$)|test_regress/t/t_preproc\.v|install-sh'

Unicode_Exempt_Re = r'(Changes$|CONTRIBUTORS$|LICENSES?|contributors.rst$|spelling.txt$|ci-rtlmeter-report.py)'


def get_source_files():
    git_files = test.run_capture("cd " + test.root + " && git ls-files")
    if test.verbose:
        print("MF " + git_files)
    files = {}
    for filename in git_files.split():
        if filename == '':
            continue
        files[filename] = True
    return files


def check_verilog_indent(filename, contents):
    # Check for proper spacing
    # This isn't automated as part of CI action auto-format because .out files
    # might break as a result
    indents = collections.defaultdict(int)
    is_on = True
    for line in contents.splitlines():
        if re.search(r'// verilog_format: off', line):
            is_on = False
        if re.search(r'// verilog_format: on', line):
            is_on = True
        if not is_on:
            continue
        # verilog-format puts input/output as double levels, ignore
        if re.match(r' *(input|output|inout) ', line):
            continue

        m = re.match(r'^ *', line)
        num_spaces = m.end() if m else 0
        if num_spaces > 0:
            indents[num_spaces] += 1

    # pprint(indents)
    spacing = 2
    if (indents[3] + indents[6] + indents[9]) > (indents[2] + indents[4] + indents[6]) * 2:
        spacing = 3
    elif indents[2] == 0 and indents[4] > 1:  # Hard because two levels of 2-space indents gives 4
        spacing = 4
    if spacing != 2:
        warns[filename] = "Indents should be 2 spaces per level, not what seems to be " + str(
            spacing) + "/level in: " + filename


if not os.path.exists(test.root + "/.git"):
    test.skip("Not in a git repository")

files = get_source_files()

warns = {}
fcount = 0
for filename in sorted(files.keys()):
    filename = os.path.join(test.root, filename)
    if not os.path.exists(filename):  # git file might be deleted but not yet staged
        continue
    contents = test.file_contents(filename)
    if re.search(r'(\.out|\.dat)$', filename):
        continue  # Ignore golden files
    if re.search(r'[\001\002\003\004\005\006]', contents):
        continue  # Ignore binary files
    if contents != "" and contents[-1] != "\n":
        contents += "\n"
        warns[filename] = "Missing trailing newline (add one) in: " + filename
    if "\r" in contents:
        contents = re.sub(r'\r', '', contents)
        warns[filename] = "Carriage returns (remove them) in: " + filename
    if "\f" in contents:
        contents = re.sub(r'\f', '', contents)
        warns[filename] = "Form-feeds (remove them) in: " + filename
    if "\t" in contents and not re.search(Tabs_Exempt_Re, filename):
        warns[filename] = "Tabs (cannot --gold fix) in " + filename

    if (re.search(r'[ \t]\n', contents)
            or re.search(r'\n\n+$', contents)):  # Regexp repeated below
        eol_ws_exempt = ('spelling.txt' in filename or '/gtkwave/' in filename)
        if eol_ws_exempt:
            continue
        if 'HARNESS_UPDATE_GOLDEN' in os.environ:
            changes = False
            (contents, n) = re.subn(r'[ \t]+\n', '\n', contents)
            if n:
                changes = True
            if not eol_ws_exempt:
                (contents, n) = re.subn(r'\n\n+$', '\n', contents)
                if n:
                    changes = True
            if not changes:
                continue
            warns[filename] = "Updated whitespace at " + filename
            test.write_wholefile(filename, contents)
            continue

        lineno = 0
        for line in contents.splitlines():
            lineno += 1
            if re.search(r'\s$', line):
                warns[filename] = "Trailing whitespace at " + filename + ":" + str(lineno)
                warns[filename] += " (last character is ASCII " + str(ord(line[-1])) + ")"

        if not eol_ws_exempt and re.search(r'\n\n+$', contents):  # Regexp repeated above
            warns[filename] = "Trailing newlines at EOF in: " + filename

    # Unicode checker; should this be done in another file?
    # No way to auto-fix.
    unicode_exempt = re.search(Unicode_Exempt_Re, filename)
    m = re.search(r'(([^ \t\r\n\x20-\x7e]).*)', contents)
    if not unicode_exempt and m:
        warns[filename] = "Warning: non-ASCII contents '" + m.group(2) + "' at '" + m.group(
            1) + "' in: " + filename

    if re.search(r'\.s?vh?$', filename):
        check_verilog_indent(filename, contents)

    fcount += 1

if fcount < 50:
    test.error("Too few source files found")

if len(warns):
    # First warning lists everything as that's shown in the driver summary
    msg = ""
    if 'HARNESS_UPDATE_GOLDEN' in os.environ:
        msg += "Updated files with whitespace errors: " + ' '.join(sorted(warns.keys())) + "\n"
    else:
        msg += "Files have whitespace errors: " + ' '.join(sorted(warns.keys())) + "\n"
        msg += "To auto-fix (some): HARNESS_UPDATE_GOLDEN=1 {command} or --golden\n"
        msg += "If change any Verilog then remember to update .out files too (with --golden)\n"
    for filename in sorted(warns.keys()):
        msg += warns[filename] + "\n"
    test.error(msg)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2025 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.sim_time = 100000

test.scenarios('simulator')

test.compile(verilator_flags2=["--stats"])

if test.vlt or test.vltmt:
    test.file_grep(test.stats, r"Scheduling, 'act' extra triggers\s+(\d+)", 1)
    test.file_grep(test.stats, r"Scheduling, 'act' pre triggers\s+(\d+)", 1)
    test.file_grep(test.stats, r"Scheduling, 'act' sense triggers\s+(\d+)", 228)

test.execute()

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2026 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('simulator')
test.top_filename = "t/t_to_string_emitted_class_hierarchy.v"

obj_type = "Derived7"
printed_from_type = "Derived7"
obj_type2 = "Derived9"
printed_from_type2 = "Derived9"
expected_to_string_classes = ["Base", "Derived1", "IClass1", "Derived7", "Derived8", "Derived9"]

test.compile(verilator_flags2=[
    "--stats", f"-DOBJ_TYPE={obj_type}", f"-DPRINTED_FROM_TYPE={printed_from_type}",
    f"-DOBJ_TYPE2={obj_type2}", f"-DPRINTED_FROM_TYPE2={printed_from_type2}"
])
for class_name in expected_to_string_classes:
    test.file_grep(
        f"{test.obj_dir}/{test.vm_prefix}___024unit__03a__03a{class_name}__Vclpkg__0.cpp",
        f"std::string {test.vm_prefix}___024unit__03a__03a{class_name}::to_string()")
test.file_grep(test.stats, r"Optimizations, Class ToString emitted\s+(\d+)",
               len(expected_to_string_classes))
test.execute()
test.files_identical(test.obj_dir + "/vlt_sim.log", test.golden_filename, "logfile")

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: RTLMeter cases to run in the rtlmeter.yml CI workflow
#
# SPDX-FileCopyrightText: 2026 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

###############################################################################
# Defines the RTLMeter cases to run in CI. This is consumed by the 'start' job
# in rtlmeter.yml, which captures the printed JSON and feeds it to the run job
# matrices.
###############################################################################

import argparse
import json

# Cases to run for scheduled runs, keyyed by the 'run tag' used in rtlmeter.yml
# The associated priority is roughly the number of minutes it takes to run the
# job, the emitted job matrix will try to start longest jobs first.

# Cases to run for non-PR jobs (e.g.: nightly schedule)
# fmt: off
CASES = {
    "gcc": [
        ("BlackParrot:1x1:*",                    5),
        ("BlackParrot:2x2:*",                    14),
        ("BlackParrot:4x4:*",                    25),
        ("Caliptra:default:*",                   19),
        ("NVDLA:*",                              29),
        ("OpenPiton:1x1:*",                      4),
        ("OpenPiton:2x2:*",                      7),
        ("OpenPiton:4x4:*",                      10),
        ("OpenTitan:*",                          16),
        ("Servant:qerv:*",                       2),
        ("Servant:serv:*",                       2),
        ("VeeR-EH1:asic:*",                      4),
        ("VeeR-EH1:default:*",                   6),
        ("VeeR-EH1:hiperf:*",                    5),
        ("VeeR-EH2:asic:*",                      13),
        ("VeeR-EH2:default:*",                   6),
        ("VeeR-EH2:hiperf:*",                    12),
        ("VeeR-EL2:asic:*",                      4),
        ("VeeR-EL2:default:*",                   4),
        ("VeeR-EL2:hiperf:*",                    4),
        ("Vortex:mini:*",                        3),
        ("Vortex:sane:*",                        6),
        ("XiangShan:default-chisel3:* !*:linux", 31),
        ("XiangShan:default-chisel6:* !*:linux", 28),
        ("XiangShan:mini-chisel3:* !*:linux",    13),
        ("XiangShan:mini-chisel6:* !*:linux",    15),
        ("XuanTie-C906:*",                       5),
        ("XuanTie-C910:*",                       9),
        ("XuanTie-E902:*",                       5),
        ("XuanTie-E906:*",                       6),
    ],
    "clang": [
        ("BlackParrot:1x1:*",                    5),
        ("BlackParrot:2x2:*",                    9),
        ("BlackParrot:4x4:*",                    12),
        ("Caliptra:default:*",                   16),
        ("NVDLA:*",                              28),
        ("OpenPiton:1x1:*",                      5),
        ("OpenPiton:2x2:*",                      6),
        ("OpenPiton:4x4:*",                      8),
        ("OpenTitan:*",                          10),
        ("Servant:qerv:*",                       17),
        ("Servant:serv:*",                       15),
        ("VeeR-EH1:asic:*",                      6),
        ("VeeR-EH1:default:*",                   8),
        ("VeeR-EH1:hiperf:*",                    5),
        ("VeeR-EH2:asic:*",                      9),
        ("VeeR-EH2:default:*",                   8),
        ("VeeR-EH2:hiperf:*",                    7),
        ("VeeR-EL2:asic:*",                      4),
        ("VeeR-EL2:default:*",                   5),
        ("VeeR-EL2:hiperf:*",                    5),
        ("Vortex:mini:*",                        3),
        ("Vortex:sane:*",                        5),
        ("XiangShan:default-chisel3:* !*:linux", 21),
        ("XiangShan:default-chisel6:* !*:linux", 19),
        ("XiangShan:mini-chisel3:* !*:linux",    8),
        ("XiangShan:mini-chisel6:* !*:linux",    8),
        ("XuanTie-C906:*",                       3),
        ("XuanTie-C910:*",                       7),
        ("XuanTie-E902:*",                       7),
        ("XuanTie-E906:*",                       7),
    ],
    "gcc-hier": [
        ("BlackParrot:1x1:*",    4),
        ("BlackParrot:2x2:*",    16),
        ("BlackParrot:4x4:*",    25),
        ("NVDLA:*",              25),
        ("OpenPiton:16x16:dhry", 16),
        ("OpenPiton:1x1:*",      4),
        ("OpenPiton:2x2:*",      4),
        ("OpenPiton:4x4:*",      4),
        ("OpenPiton:8x8:*",      6),
        ("XuanTie-C910:*",       7),
    ],
}
# fmt: on

# Cases to run for PR jobs
# fmt: off
CASES_PR = {
    "gcc": [
        ("BlackParrot:1x1:*",                    5),
        ("BlackParrot:4x4:*",                    25),
        ("Caliptra:default:*",                   19),
        ("NVDLA:*",                              29),
        ("OpenPiton:1x1:*",                      4),
        ("OpenPiton:4x4:*",                      10),
        ("OpenTitan:*",                          16),
        ("Servant:qerv:*",                       2),
        ("Servant:serv:*",                       2),
        ("VeeR-EH1:asic:*",                      4),
        ("VeeR-EH1:default:*",                   6),
        ("VeeR-EH1:hiperf:*",                    5),
        ("VeeR-EH2:asic:*",                      13),
        ("VeeR-EH2:default:*",                   6),
        ("VeeR-EH2:hiperf:*",                    12),
        ("VeeR-EL2:asic:*",                      4),
        ("VeeR-EL2:default:*",                   4),
        ("VeeR-EL2:hiperf:*",                    4),
        ("Vortex:mini:*",                        3),
        ("Vortex:sane:*",                        6),
        ("XiangShan:default-chisel3:* !*:linux", 31),
        ("XiangShan:default-chisel6:* !*:linux", 28),
        ("XiangShan:mini-chisel3:* !*:linux",    13),
        ("XiangShan:mini-chisel6:* !*:linux",    15),
        ("XuanTie-C906:*",                       5),
        ("XuanTie-C910:*",                       9),
        ("XuanTie-E902:*",                       5),
        ("XuanTie-E906:*",                       6),
    ],
    "clang": [
        ("BlackParrot:1x1:*",                    5),
        ("BlackParrot:4x4:*",                    12),
        ("Caliptra:default:*",                   16),
        ("NVDLA:*",                              28),
        ("OpenPiton:1x1:*",                      5),
        ("OpenPiton:4x4:*",                      8),
        ("OpenTitan:*",                          10),
        ("Servant:qerv:*",                       17),
        ("Servant:serv:*",                       15),
        ("VeeR-EH1:asic:*",                      6),
        ("VeeR-EH1:default:*",                   8),
        ("VeeR-EH1:hiperf:*",                    5),
        ("VeeR-EH2:asic:*",                      9),
        ("VeeR-EH2:default:*",                   8),
        ("VeeR-EH2:hiperf:*",                    7),
        ("VeeR-EL2:asic:*",                      4),
        ("VeeR-EL2:default:*",                   5),
        ("VeeR-EL2:hiperf:*",                    5),
        ("Vortex:mini:*",                        3),
        ("Vortex:sane:*",                        5),
        ("XiangShan:default-chisel3:* !*:linux", 21),
        ("XiangShan:default-chisel6:* !*:linux", 19),
        ("XiangShan:mini-chisel3:* !*:linux",    8),
        ("XiangShan:mini-chisel6:* !*:linux",    8),
        ("XuanTie-C906:*",                       3),
        ("XuanTie-C910:*",                       7),
        ("XuanTie-E902:*",                       7),
        ("XuanTie-E906:*",                       7),
    ],
    "gcc-hier": [
        ("BlackParrot:1x1:*",    4),
        ("BlackParrot:4x4:*",    25),
        ("NVDLA:*",              25),
        ("OpenPiton:16x16:dhry", 16),
        ("OpenPiton:1x1:*",      4),
        ("OpenPiton:4x4:*",      4),
        ("OpenPiton:8x8:*",      6),
        ("XuanTie-C910:*",       7),
    ],
}
# fmt: on

parser = argparse.ArgumentParser(
    description="Print the RTLMeter CI cases as JSON, keyed by run tag")
parser.add_argument("--event-name",
                    required=True,
                    help="GitHub event name that triggered the workflow")
parser.add_argument(
    "--ci-testing",
    action="store_true",
    help="Print only the shortest (lowest priority) case per tag, to reduce CI testing time")
args = parser.parse_args()

# Pull requests use the reduced 'CASES_PR' set, other events (e.g.: the nightly
# schedule) use the full 'CASES' set.
is_pr = args.event_name == "pull_request"
selected = CASES_PR if is_pr else CASES

cases = {}
for tag, entries in selected.items():
    # Highest priority (longest) first
    ordered = sorted(entries, key=lambda cp: cp[1], reverse=True)
    # While tweaking the CI, keep only the shortest (lowest priority) case to minimise churn time
    if args.ci_testing:
        ordered = ordered[-1:]
    # Case filters appended to every case for this tag/event
    suffix = ""
    # Filters out the non-hierarchical case variants for the hierarchical job
    if tag == "gcc-hier":
        suffix += " !-hier"
    # Filter out hello tests on pull requests, these just add measurement noise
    if is_pr:
        suffix += " !*:hello"
    cases[tag] = [case + suffix for case, _ in ordered]
print(json.dumps(cases, separators=(",", ":")))



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('dist')

if not os.path.exists(test.root + "/.git"):
    test.skip("Not in a git repository")

### Must trim output before and after our file list
files = test.run_capture("cd " + test.root + " && git ls-files --exclude-standard")
if test.verbose:
    print("ST " + files)

names = {}
files = re.sub(r'\s+', ' ', files)

regex = r'(FIX[M]E|BO[Z]O)'
for filename in files.split():
    if re.search(regex, filename):
        names[filename] = True
    filename = os.path.join(test.root, filename)
    if not os.path.exists(filename):
        continue
    wholefile = test.file_contents(filename)
    if re.search(regex, wholefile):
        names[filename] = True

if len(names):
    test.error("Files with FIX" + "MEs: " + ' '.join(sorted(names.keys())))

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')
test.top_filename = "t/t_class_static_method.v"

# This test makes randomly named .cpp/.h files, which tend to collect, so remove them first
for filename in (glob.glob(test.obj_dir + "/*_PS*.cpp") + glob.glob(test.obj_dir + "/*_PS*.h") +
                 glob.glob(test.obj_dir + "/*.d")):
    test.unlink_ok(filename)

test.compile(verilator_flags2=["--protect-ids", "--protect-key SECRET_KEY"])

test.execute()

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt_all')
test.top_filename = "t/t_flag_csplit.v"


def check_no_splits():
    for filename in test.glob_some(test.obj_dir + "/*.cpp"):
        filename = re.sub(r'__024root', '', filename)
        if re.search(r'__[1-9]', filename):
            test.error("Split file found: " + filename)


def check_all_file():
    for filename in test.glob_some(test.obj_dir + "/*.cpp"):
        if re.search(r'__ALL.cpp', filename):
            return


def check_gcc_flags(filename):
    # Coverage collection alters optimization flags
    if test.have_dev_gcov:
        return
    with open(filename, 'r', encoding="utf8") as fh:
        for line in fh:
            line = line.rstrip()
            if test.verbose:
                print(":log: " + line)
            if re.search(r'\.cpp', line) and re.search('-O0', line):
                test.error("File built as slow (should be in __ALL.cpp) : " + line)


# This rule requires GNU make > 4.1 (or so, known broken in 3.81)
#%__Slow.o: %__Slow.cpp
#        $(OBJCACHE) $(CXX) $(CXXFLAGS) $(CPPFLAGS) $(OPT_SLOW) -c -o $@ $<
if not test.make_version or float(test.make_version) < 4.1:
    test.skip("Test requires GNU Make version >= 4.1")

test.compile(v_flags2=["--trace-vcd --output-split 0 --exe ../" + test.main_filename],
             verilator_make_gmake=False)

# We don't use the standard test_regress rules, as want to test the rules
# properly build
test.run(logfile=test.obj_dir + "/vlt_gcc.log",
         tee=test.verbose,
         cmd=[
             os.environ["MAKE"], "-C " + test.obj_dir, "-f " + test.vm_prefix + ".mk", "-j 4",
             "VM_PREFIX=" + test.vm_prefix, "TEST_OBJ_DIR=" + test.obj_dir,
             "CPPFLAGS_DRIVER=-D" + test.name.upper(),
             ("CPPFLAGS_DRIVER2=-DTEST_VERBOSE=1" if test.verbose else ""), "OPT_FAST=-O2",
             "OPT_SLOW=-O0"
         ])

test.execute()

# Never spliting, so should set VM_PARALLEL_BUILDS to 0 by default
test.file_grep(test.obj_dir + "/" + test.vm_prefix + "_classes.mk", r'VM_PARALLEL_BUILDS\s*=\s*0')
check_no_splits()
check_all_file()
check_gcc_flags(test.obj_dir + "/vlt_gcc.log")

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: FSM case(state_d) overwritten-default warning test
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2026 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')

# A canonical `state_d = state_q` default followed by another top-level write
# to `state_d` before `case (state_d)` is legal RTL, but it is not the narrow
# Phase 1 shape we support. Warn and reject rather than silently skipping it.
test.compile(verilator_flags2=["--cc --coverage"],
             fails=True,
             expect_filename=test.golden_filename)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap
import collections
import math

test.scenarios('simulator')
test.top_filename = test.obj_dir + "/t_gate_tree.v"
test.cycles = (1000000 if test.benchmark else 100)
test.sim_time = test.cycles * 10 + 1000

width = 64 * int(test.getenv_def("VERILATOR_TEST_WIDTH", "4"))
nvars = 64


def gen(filename):
    with open(filename, 'w', encoding="utf8") as fh:
        fh.write("// Generated by t_gate_tree.py\n")
        fh.write("module t (clk);\n")
        fh.write("  input clk;\n")
        fh.write("\n")
        fh.write("  integer cyc=0;\n")
        fh.write("  reg reset;\n")
        fh.write("\n")

        tree = collections.defaultdict(dict)
        fanin = 8
        stages = int(math.log(nvars) / math.log(fanin) + 0.99999) + 1
        result = 0
        for n in range(0, nvars):
            result += max(n, 1)
            if 0 not in tree:
                tree[0] = {}
            if n not in tree[0]:
                tree[0][n] = {}
            tree[0][n][n] = True
            nl = n
            for stage in range(1, stages):
                lastn = nl
                nl = int(nl / fanin)
                if stage not in tree:
                    tree[stage] = {}
                if nl not in tree[stage]:
                    tree[stage][nl] = {}
                tree[stage][nl][lastn] = True

        # pprint(tree)

        fh.write("\n")
        workingset = 0
        for stage in sorted(tree.keys()):
            for n in sorted(tree[stage].keys()):
                fh.write("   reg [" + str(width - 1) + ":0] v" + str(stage) + "_" + str(n) + ";\n")
                workingset += int(width / 8 + 7)

        fh.write("\n")
        fh.write("   always @ (posedge clk) begin\n")
        fh.write("      cyc <= cyc + 1;\n")
        fh.write("`ifdef TEST_VERBOSE\n")
        fh.write("         $write(\"[%0t] rst=%0x  v0_0=%0x  v1_0=%0x  result=%0x\\n\""
                 ", $time, reset, v0_0, v1_0, v" + str(stages - 1) + "_0);\n")
        fh.write("`endif\n")
        fh.write("      if (cyc==0) begin\n")
        fh.write("         reset <= 1;\n")
        fh.write("      end\n")
        fh.write("      else if (cyc==10) begin\n")
        fh.write("         reset <= 0;\n")
        fh.write("      end\n")
        fh.write("`ifndef SIM_CYCLES\n")
        fh.write(" `define SIM_CYCLES 99\n")
        fh.write("`endif\n")
        fh.write("      else if (cyc==`SIM_CYCLES) begin\n")
        fh.write("         if (v" + str(stages - 1) + "_0 != " + str(width) + "'d" + str(result) +
                 ") $stop;\n")
        fh.write("         $write(\"VARS=" + str(nvars) + " WIDTH=" + str(width) + " WORKINGSET=" +
                 str(int(workingset / 1024)) + "KB\\n\");\n")
        fh.write('         $write("*-* All Finished *-*\\n");' + "\n")
        fh.write('         $finish;' + "\n")
        fh.write("      end\n")
        fh.write("   end\n")

        fh.write("\n")
        for n in range(0, nvars):
            fh.write("   always @ (posedge clk)" + " v0_" + str(n) + " <= reset ? " + str(width) +
                     "'d" + str(max(n, 1)) + " : v0_" + str((int(n / fanin) * fanin) +
                                                            ((n + 1) % fanin)) + ";\n")

        for stage in sorted(tree.keys()):
            if stage == 0:
                continue
            fh.write("\n")
            for n in sorted(tree[stage].keys()):
                fh.write("   always @ (posedge clk) v" + str(stage) + "_" + str(n) + " <=")
                op = ""
                for ni in sorted(tree[stage][n].keys()):
                    fh.write(op + " v" + str(stage - 1) + "_" + str(ni))
                    op = " +"
                fh.write(";\n")

        fh.write("endmodule\n")


gen(test.top_filename)

test.compile(v_flags2=["+define+SIM_CYCLES=" + str(test.cycles)],
             verilator_flags2=["--stats --x-assign fast --x-initial fast", "-Wno-UNOPTTHREADS"])

test.execute(all_run_flags=[
    "+verilator+prof+exec+start+100",
    " +verilator+prof+exec+window+2",
    " +verilator+prof+exec+file+" + test.obj_dir + "/profile_exec.dat",
    " +verilator+prof+vlt+file+" + test.obj_dir + "/profile.vlt"])   # yapf:disable

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('simulator')
test.top_filename = "t/t_sys_sformat.v"

test.compile(
    # Avoid inlining our simple example, to make sure verilated.h works right
    verilator_flags2=["-O0"])

if re.search(r'clang', test.cxx_version):
    test.skip("Known clang bug")
    #Here:   if (VL_UNLIKELY(VL_NEQ_W(12, __Vtemp1, vlSymsp->TOP__t.__PVT__str)))

test.execute()

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')

test.setenv('FOOBARTEST', "gotit")

test.run(cmd=[os.environ["VERILATOR_ROOT"] + "/bin/verilator --getenv FOOBARTEST"],
         logfile=test.compile_log_filename,
         verilator_run=True)

test.file_grep(test.compile_log_filename, r'gotit')

for var in [
        'MAKE', 'PERL', 'PYTHON3', 'SYSTEMC', 'SYSTEMC_ARCH', 'SYSTEMC_INCLUDE', 'SYSTEMC_LIBDIR',
        'VERILATOR_ROOT'
]:
    test.run(cmd=[os.environ["VERILATOR_ROOT"] + "/bin/verilator", "--getenv", var],
             logfile=test.obj_dir + "/simx.log",
             verilator_run=True)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')

test.compile(
    verilator_flags2=["--stats", "--build", "--gate-stmts", "10000", "--expand-limit", "128"])

test.file_grep(test.stats, r'Optimizations, FuncOpt concat trees balanced\s+(\d+)', 3)
test.file_grep(test.stats, r'Optimizations, FuncOpt concat splits\s+(\d+)', 67)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Multiple Model Test Module
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt_all')

test.compile(
    make_top_shell=False,
    make_main=False,
    # link threads library, add custom .cpp code, add tracing & coverage support
    verilator_flags2=["--exe", test.pli_filename, "--trace-vcd --coverage -cc"],
    threads=1,
    make_flags=['CPPFLAGS_ADD=-DVL_NO_LEGACY'])

test.execute()
test.files_identical_sorted(test.obj_dir + "/coverage_top0.dat",
                            "t/t_wrapper_context__top0.dat.out")
test.files_identical_sorted(test.obj_dir + "/coverage_top1.dat",
                            "t/t_wrapper_context__top1.dat.out")

test.vcd_identical(test.obj_dir + "/trace0.vcd", "t/t_wrapper_context__trace0.vcd.out")
test.vcd_identical(test.obj_dir + "/trace1.vcd", "t/t_wrapper_context__trace1.vcd.out")

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap
from pathlib import Path

test.scenarios('simulator')

test.compile(verilator_flags2=['--cc', '--coverage-line'])

test.execute()

test.run(cmd=[
    os.environ["VERILATOR_ROOT"] + "/bin/verilator_coverage",
    "--annotate-points",
    "--annotate",
    test.obj_dir + "/annotated",
    test.obj_dir + "/coverage.dat",
],
         verilator_run=True)

top = Path(test.top_filename)
test.files_identical(test.obj_dir + f"/annotated/{top.name}", test.golden_filename)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('simulator')
test.top_filename = "t/t_func_dotted.v"

out_filename = test.obj_dir + "/V" + test.name + ".tree.json"

test.compile(v_flags2=["--no-json-edit-nums", "t/t_func_dotted_inl1.vlt"])

if test.vlt_all:
    test.file_grep_not(out_filename, r'"ma0"')
    test.file_grep_not(out_filename, r'"mb0"')
    test.file_grep_not(out_filename, r'"mc0"')

test.execute()

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt_all')
test.pli_filename = "t/t_trace_public_sig.cpp"
test.top_filename = "t/t_trace_public.v"
test.golden_filename = "t/t_trace_public.out"

out_filename = test.obj_dir + "/V" + test.name + ".tree.json"

test.compile(make_top_shell=False,
             make_main=False,
             v_flags2=[
                 "--trace-vcd --exe", test.pli_filename,
                 test.t_dir + "/t_trace_public_sig.vlt --no-json-edit-nums"
             ])

if test.vlt_all:
    test.file_grep(
        out_filename,
        r'{"type":"VAR","name":"GSR",.*"loc":"\w,47:[^"]*",.*"origName":"GSR",.*"isSigPublic":true,.*"dtypeName":"logic",.*"isSigUserRdPublic":true.*"isSigUserRWPublic":true'
    )

test.execute()

test.vcd_identical(test.trace_filename, test.golden_filename)

# vcd_identical doesn't detect "$var a.b;" vs "$scope module a; $var b;"
test.file_grep(test.trace_filename, r'module glbl')

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3,
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2026 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import os

import vltest_bootstrap

test.scenarios('simulator')

test.compile(top_filename="t/t_cover_per_instance.v",
             verilator_flags2=[
                 '--binary',
                 '--coverage-line',
                 '--coverage-per-instance',
                 '--top-module',
                 'tb',
                 '--timing',
             ])

test.execute(all_run_flags=[" +verilator+coverage+file+" + test.obj_dir + "/coverage.dat"])

cov = test.obj_dir + "/coverage.dat"

test.files_identical(cov, "t/t_cover_per_instance.dat.out")

test.run(cmd=[
    os.environ["VERILATOR_ROOT"] + "/bin/verilator_coverage",
    "--annotate-points",
    "--annotate",
    test.obj_dir + "/annotated-points",
    cov,
],
         verilator_run=True)

test.files_identical(test.obj_dir + "/annotated-points/t_cover_per_instance.v",
                     "t/t_cover_per_instance.out")

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('simulator')
test.top_filename = test.obj_dir + "/t_unicode.v"


def c(code):
    # Appease https://www.virustotal.com NANO-Antivirus gives Trojan.Script.Vbs-heuristic flag
    return eval("c" + "h" + "r(" + str(code) + ")")  # pylint: disable=eval-used


# Greek Hi
hi = "Greek: " + c(0xce) + c(0xb3) + c(0xce) + c(0xb5) + c(0xce) + c(0xb9) + c(0xce) + c(0xb1)


def gen(filename):
    with open(filename, 'w', encoding="latin-1") as fh:
        fh.write(c(0xEF))
        fh.write(c(0xBB))
        fh.write(c(0xBF))  # BOM
        fh.write("// Bom\n")
        fh.write("// Generated by t_unicode.py\n")
        fh.write("module t;\n")
        fh.write("   // " + hi + "\n")
        fh.write("   initial begin\n")
        fh.write("      $write(\"" + hi + "\\n\");\n")
        fh.write("      $write(\"*-* All Finished *-*\\n\");\n")
        fh.write("      $finish;\n")
        fh.write("   end\n")
        fh.write("endmodule\n")


gen(test.top_filename)

test.compile()

test.execute()

test.file_grep(test.run_log_filename, hi)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Primitive C++ style checker
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('dist')

Doc_Waivers = [
    '+verilator+prof+threads+file+',  # Deprecated
    '+verilator+prof+threads+start+',  # Deprecated
    '+verilator+prof+threads+window+',  # Deprecated
    '-clk',  # Deprecated
    '-lineno',  # Deprecated
    '-order-clock-delay',  # Deprecated
    '-pp-comments',  # Deprecated
    '-prof-threads',  # Deprecated
    '-structs-packed',  # Deprecated
    '-trace-threads',  # Deprecated
    '-xml-only',  # Removed
    '-xml-output',  # Removed
]

Test_Waivers = [
    # Covered:
    '-G',  # Covered; other text fallows option letter
    '-O',  # Covered; other text fallows option letter
    '-U',  # Covered; other text fallows option letter
    '-gdb',  # Covered: no way to test, part of --gdbbt
    '-rr',  # Not testing; not requiring rr installation
    # Need testing:
    '-fconst',  # TODO breaks due to some needed V3Const steps
]

Sums = {}
Docs = {}
Srcs = {}
Tests = {}


def read_sums():
    for filename in test.glob_some(test.root + "/bin/*"):
        with open(filename, "r", encoding="latin-1") as fh:
            on = False
            lineno = 0
            for line in fh:
                lineno += 1
                line = line.rstrip()
                m1 = re.search(r'^\s+((-|\+)+[^ ]+)', line)
                m2 = re.search(r"parser.add_argument\('((-|\+)[^']+)'", line)
                if re.search(r'ARGUMENT SUMMARY', line):
                    on = True
                elif re.search(r'=head1', line):
                    on = False
                elif on and m1:
                    opt = opt_clean(m1.group(1))
                    key = opt_key(opt)
                    if test.verbose:
                        print("A '" + opt + "' " + line)
                    Sums[key] = filename + ":" + str(lineno) + ": " + opt
                elif m2:
                    opt = opt_clean(m2.group(1))
                    key = opt_key(opt)
                    if test.verbose:
                        print("A '" + opt + "' " + line)
                    Sums[key] = filename + ":" + str(lineno) + ": " + opt


def read_docs():
    for filename in test.glob_some(test.root + "/docs/guide/*.rst"):
        with open(filename, "r", encoding="latin-1") as fh:
            lineno = 0
            for line in fh:
                lineno += 1
                line = line.rstrip()
                m = re.search(r'option:: ((-|\+)+[^ `]+)', line)
                if not m:
                    m = re.search(r':vlopt:`[^`]+ <([^>]+)>', line)
                if not m:
                    m = re.search(r':vlopt:`((-|\+)+[^ `]+)', line)
                if m:
                    opt = opt_clean(m.group(1))
                    key = opt_key(opt)
                    if test.verbose:
                        print("D '" + opt + "' " + line)
                    Docs[key] = filename + ":" + str(lineno) + ": " + opt


def read_srcs():
    filename = "bin/verilator --debug-options"
    opts = test.run_capture("perl " + os.environ["VERILATOR_ROOT"] + "/bin/verilator" +
                            " --debug-options 2>&1")
    lineno = 0
    for line in opts.split("\n"):
        lineno += 1
        m1 = re.search(r'OPTION: "([^"]+)"', line)
        if m1:
            opt = opt_clean(m1.group(1))
            key = opt_key(opt)
            if re.match(r'-Werror-[A-Z ]', opt):
                continue
            if re.match(r'-Wno-[A-Z ]', opt):
                continue
            if re.match(r'-Wwarn-[A-Z ]', opt):
                continue
            if test.verbose:
                print("S '" + opt + "' " + line)
            Srcs[key] = filename + ":" + str(lineno) + ": " + opt
    if len(Srcs) < 5:
        test.error("Didn't parse any options")
    return Srcs


def read_tests():
    filename = "test_regress/t/*.py"
    for filename in (test.glob_some(test.root + "/test_regress/t/*.py")):
        if "t_dist_docs_options" in filename:  # Avoid our own suppressions
            continue
        with open(filename, 'r', encoding="latin-1") as fh:
            lineno = 0
            for line in fh:
                lineno += 1
                line = line.lstrip().rstrip()
                for opt in re.findall(r'[-+]+[-+a-zA-Z0-9]+', line):
                    opt = opt_clean(opt)
                    key = opt_key(opt)
                    if test.verbose:
                        print("T '" + opt + "' " + line)
                    Tests[key] = filename + ":" + str(lineno) + ": " + opt
                    # For e.g. +verilator+seed+<something> and -dumpi-<#>
                    pos = max(opt.rfind('+'), opt.rfind('-'))
                    if pos > 1:
                        subkey = opt[:pos + 1]
                        if test.verbose:
                            print("t '" + subkey + "' " + line)
                        Tests[subkey] = filename + ":" + str(lineno) + ": " + opt


def opt_clean(opt):
    opt = re.sub(r'--', '-', opt)
    opt = re.sub(r'<.*', '', opt)
    opt = re.sub(r'\\', '', opt)
    return opt


def opt_key(opt):
    opt = opt_clean(opt)
    opt = re.sub(r'^-fno-', '-f', opt)
    opt = re.sub(r'^-no-', '-', opt)
    return opt


if not os.path.exists(test.root + "/.git"):
    test.skip("Not in a git repository")

read_sums()
read_docs()
read_srcs()
read_tests()

both = {}
both.update(Sums)
both.update(Docs)
both.update(Srcs)

doc_waiver = {k: 1 for k in Doc_Waivers}

for opt in sorted(both.keys()):
    if opt in doc_waiver:
        continue

    is_in = []
    not_in = []
    summ = None
    if opt in Sums:
        summ = Sums[opt]
        is_in.append("summary " + summ)
    else:
        if not re.match(r'-f', opt):
            not_in.append("ARGUMENT SUMMARY in bin/verilator or bin/verilator_coverage")

    doc = None
    if opt in Docs:
        doc = Docs[opt]
        is_in.append("documentation " + doc)
    else:
        not_in.append("documentation in docs/guide/exe_*.rst")

    src = None
    if opt in Srcs:
        src = Srcs[opt]
        is_in.append("sources " + src)
    # Ok if not in sources

    if opt in Tests:
        if opt in Test_Waivers:
            print("Unnecessary Test_Waiver for option: '" + opt + "'")

    else:
        if opt in Test_Waivers:
            is_in.append("Test_Waiver for test_regress/t/*.py")
        else:
            not_in.append("uncovered in test_regress/t/*.py")

    if not_in:
        test.error_keep_going("Option '" + opt + "' has inconsistent references\n" +
                              "  Missing in " + "\n  Missing in ".join(not_in) + "\n  Found in " +
                              "\n  Found in ".join(is_in))

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')

test.lint(v_flags=["--dump-inputs -Wno-MULTITOP t/t_debug_inputs.vlt t/t_debug_inputs_b.v"])

test.file_grep(test.obj_dir + "/V" + test.name + "__inputs.vpp", r'module t_debug_inputs;')
test.file_grep(test.obj_dir + "/V" + test.name + "__inputs.vpp", r'module t_debug_inputs_a;')
test.file_grep(test.obj_dir + "/V" + test.name + "__inputs.vpp", r'module t_debug_inputs_b;')

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')

# This doesn't use the general compile rule as we want to make sure we form
# prefix properly using post-escaped identifiers
test.run(cmd=[
    os.environ["VERILATOR_ROOT"] + "/bin/verilator",
    "--binary",
    "--Mdir " + test.obj_dir + "/t_mod_dot",
    't/t_mod_dot.v',
],
         verilator_run=True)

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('dist')

if not os.path.exists(os.environ["VERILATOR_ROOT"] + "/bin/verilator_difftree"):
    test.skip("No verilator_difftree available")

test.run(cmd=[
    "cd " + test.obj_dir + " && " + os.environ["VERILATOR_ROOT"] + "/bin/verilator_difftree",
    test.t_dir + "/t_difftree.a.tree", test.t_dir + "/t_difftree.b.tree > diff.log"
],
         fails=1)  # Testing mismatch, so exit code 1

test.files_identical(test.obj_dir + "/diff.log", test.golden_filename, 'logfile')

test.passes()



#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('simulator')
test.top_filename = "t/t_langext_2.v"

# This is a compile only test.
test.compile(v_flags2=["+1800-2023ext+v"])

test.passes()
